/*
 * This file is part of MySQLConnectus by Sergio Decherchi
 * 
 * Copyright (C) 2004 Sergio Decherchi
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details (gpl.txt). You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package dbconnectivity;

import java.awt.Component;
import java.io.IOException;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.swing.JOptionPane;

import mysqlconnectus.MainFrame;
import tools.MiniParser;

/**
 * Class that filters and manages every action towards the selected dbms
 * (PROXY pattern used)
 * @version 0.3
 */
public class DatabaseSource implements DatabaseConnector {

	/** MySQL Database id string */
	final public static String MYSQL = "MySQL"; //$NON-NLS-1$

	/** Oracle Database id string */
	final public static String ORACLE = "Oracle"; //$NON-NLS-1$

	/** SqlServer Database id string */
	final public static String SQLSERVER = "SQLServer"; //$NON-NLS-1$

	/** Postgress Database id string */
	final public static String POSTGRESQL = "Postgresql"; //$NON-NLS-1$

	/** Currently choosen db */
	private String choosenDBMS;

	/** Pointer to current db Access class */
	private Object dbc;

	/** Cache */
	private Hashtable cache = null;

	/** Parent component */
	private Component parent;

	/** Logging */
	private Logger logger;
	
	/** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages"); //$NON-NLS-1$
	

	/**
	 * Default Constructor
	 * 
	 * @param type
	 *                 is the type of db,
	 * @param ancestor
	 *                 is Component father ,if null this feature is ignored
	 */
	public DatabaseSource(String type, Component ancestor) {
		parent = ancestor;
		cache = new Hashtable();
		logger = Logger.getLogger("MySQLConnectus 0.4.0 log file");
		try {
			//Open current log.txt file
			FileHandler fh = new FileHandler("log.txt"); //$NON-NLS-1$
			fh.setFormatter(new SimpleFormatter());
			logger.addHandler(fh);
			logger.info(Messages.getString("DatabaseSource.5")); //$NON-NLS-1$

		} catch (SecurityException e) {
			JOptionPane.showMessageDialog(null, Messages.getString("DatabaseSource.6"), //$NON-NLS-1$
					Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
			logger.warning(Messages.getString("DatabaseSource.6") + Messages.getString("DatabaseSource.1") //$NON-NLS-1$ //$NON-NLS-2$
					+ e.getMessage() + "\n"); //$NON-NLS-1$
			e.printStackTrace();
		} catch (IOException e) {

			JOptionPane.showMessageDialog(null, Messages.getString("DatabaseSource.0"), Messages.getString("MainFrame.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
			logger.severe(Messages.getString("DatabaseSource.0") + Messages.getString("DatabaseSource.1") + e.getMessage() //$NON-NLS-1$ //$NON-NLS-2$
					+ "\n"+e.getMessage()); //$NON-NLS-1$
			e.printStackTrace();
		}
		if (type.equals(MYSQL)) {
			dbc = Axxess.getInstance();

			if (ancestor != null) {
				((Axxess) dbc).setAncestor(ancestor);
				//And other modifications if necessary
			}
			choosenDBMS = MYSQL.concat(""); //$NON-NLS-1$
		} else if (type.equals(ORACLE)) {
			//TO DO
			choosenDBMS = ORACLE.concat(""); //$NON-NLS-1$
		} else if (type.equals(SQLSERVER)) {
			//TO DO
			choosenDBMS = SQLSERVER.concat(""); //$NON-NLS-1$
		} else if (type.equals(POSTGRESQL)) {
			//TO DO
			choosenDBMS = POSTGRESQL.concat(""); //$NON-NLS-1$
		}
	}

	/** Interface DatabaseConnector implementation */
	public String access(String user, String password, String hostname) {
		if (choosenDBMS.equals(MYSQL)) {
			long a = System.currentTimeMillis();
			String exitus = ((Axxess) dbc).access(user, password, hostname);
			if (exitus.equals(Messages.getString("Axxess.GoodConnection"))) { //$NON-NLS-1$
				long b = System.currentTimeMillis();
				logger.info(Messages.getString("DatabaseSource.21") + user +" " +Messages.getString("DatabaseSource.22") //$NON-NLS-1$ //$NON-NLS-2$
						+ hostname +" "+ Messages.getString("DatabaseSource.23") + (b - a) / 1000f + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
				if (parent!=null)
				((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.21") + user.toUpperCase()+" " //$NON-NLS-1$
						+ Messages.getString("DatabaseSource.22") + hostname +" "+ Messages.getString("DatabaseSource.23") //$NON-NLS-1$ //$NON-NLS-2$
						+ (b - a) / 1000f);
			}
			return exitus;
		} else if (choosenDBMS.equals(ORACLE)) {
		}//Do something
		else if (choosenDBMS.equals(SQLSERVER)) {
		}//Do something
		else if (choosenDBMS.equals(POSTGRESQL)) {
		}//Do something

		return null;
	}

	/** Interface DatabaseConnector implementation */
	public BufferedResultSet executeQuery(String query) {
		if (choosenDBMS.equals(MYSQL)) {
			MiniParser mp = new MiniParser();
			query = mp.setStandard(query);
			if (!query.startsWith("show")) { //$NON-NLS-1$
				return checkCache(query, dbc);
			}
			return ((Axxess) dbc).executeQuery(query);
		}

		else if (choosenDBMS.equals(ORACLE)) {
		}//Do something
		else if (choosenDBMS.equals(SQLSERVER)) {
		}//Do something
		else if (choosenDBMS.equals(POSTGRESQL)) {
		}//Do something

		return null;
	}

	/** Interface DatabaseConnector implementation */
	public boolean executeScript(String filename) {
		if (choosenDBMS.equals(MYSQL)) {
			long a = System.currentTimeMillis();
			boolean correctness = ((Axxess) dbc).executeScript(filename);
			long b = System.currentTimeMillis();
			if (correctness) {
				logger.info(Messages.getString("DatabaseSource.21") //$NON-NLS-1$
						+ ((Axxess) dbc).getConnectionData().getUsername()
								.toUpperCase()+" "
						+ Messages.getString("DatabaseSource.30") //$NON-NLS-1$
						+ ((Axxess) dbc).getConnectionData().getHost()
						+ Messages.getString("DatabaseSource.23") + (b - a) / 1000f + Messages.getString("DatabaseSource.32") //$NON-NLS-1$ //$NON-NLS-2$
						+ "" + filename + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
				((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.21") //$NON-NLS-1$
						+ ((Axxess) dbc).getConnectionData().getUsername()
								.toUpperCase()+" "
						+ Messages.getString("DatabaseSource.30") //$NON-NLS-1$
						+ ((Axxess) dbc).getConnectionData().getHost()+" "
						+ Messages.getString("DatabaseSource.23") + (b - a) / 1000f); //$NON-NLS-1$
			} else {
				logger
						.warning(Messages.getString("DatabaseSource.38")); //$NON-NLS-1$
				((MainFrame) parent)
						.setLogText(Messages.getString("DatabaseSource.39")); //$NON-NLS-1$
			}
		} else if (choosenDBMS.equals(ORACLE)) {
		}//Do something
		else if (choosenDBMS.equals(SQLSERVER)) {
		}//Do something
		else if (choosenDBMS.equals(POSTGRESQL)) {
		}//Do something
		return false;
	}

	/** Interface DatabaseConnector implementation */
	public boolean executeUpdein(String query,boolean warning) {

		if (choosenDBMS.equals(MYSQL)) {
			long a = System.currentTimeMillis();
			boolean response = ((Axxess) dbc).executeUpdein(query,warning);
			String modtext = query;
			long b = System.currentTimeMillis();
			if (response) {
				invalidateCache();
			}
			query = query.toLowerCase();
			if (query.startsWith("use") && response) //$NON-NLS-1$
				query = Messages.getString("DatabaseSource.41") + query.substring(3, query.length()); //$NON-NLS-1$
			else
				query = Messages.getString("DatabaseSource.42") + (b - a) / 1000; //$NON-NLS-1$
			if (response) {
				logger.info(query + Messages.getString("DatabaseSource.43") + "" + modtext + "\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				((MainFrame) parent).setLogText(query);
			} else {
				if (warning)
				{
				logger.severe(query + Messages.getString("DatabaseSource.43") + "" + modtext + "\n" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						+ Messages.getString("DatabaseSource.49")); //$NON-NLS-1$
				((MainFrame) parent)
						.setLogText(Messages.getString("DatabaseSource.50")); //$NON-NLS-1$
				}
			}
			return response;
		} else if (choosenDBMS.equals(ORACLE)) {
		}//Do something
		else if (choosenDBMS.equals(SQLSERVER)) {
		}//Do something
		else if (choosenDBMS.equals(POSTGRESQL)) {
		}//Do something
		return false;
	}

	public ConnectionData getConnectionData() {
		return ((Axxess) dbc).getConnectionData();
	}
	/** Interface DatabaseConnector implementation */
	public boolean disconnect() {
		if (choosenDBMS.equals(MYSQL)) {
			boolean result = ((Axxess) dbc).disconnect();
			if (result) {
				logger.info(Messages.getString("DatabaseSource.51")); //$NON-NLS-1$
				((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.52")); //$NON-NLS-1$
			} else {
				logger.warning(Messages.getString("DatabaseSource.53")); //$NON-NLS-1$
				((MainFrame) parent)
						.setLogText(Messages.getString("DatabaseSource.54")); //$NON-NLS-1$
			}
			return result;
		}

		else {
			logger.warning(Messages.getString("DatabaseSource.53")); //$NON-NLS-1$
			((MainFrame) parent)
					.setLogText(Messages.getString("DatabaseSource.54")); //$NON-NLS-1$
			return false;
		}

	}

	/** Method for invalidating cache */
	public void invalidateCache() {
		/* Invalidate cache memory */
		cache = new Hashtable();
		((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.57")); //$NON-NLS-1$
		logger.info(Messages.getString("DatabaseSource.58")); //$NON-NLS-1$
	}

	/**
	 * Check cache for an already executed query
	 * 
	 * @return A bufferedResultSet object is returned if exists , or null if not
	 *             present
	 */
	public BufferedResultSet checkCache(String query, Object dbs) {
		Axxess mysqldb = null;
		BufferedResultSet tmp = null;
		long a = System.currentTimeMillis();

		if (choosenDBMS.equals("MySQL")) { //$NON-NLS-1$
			mysqldb = (Axxess) dbs;
		}

		//System.out.println(query);
		if ((tmp = (BufferedResultSet) cache.get(query)) != null) {
			long b = System.currentTimeMillis();
			logger.info(Messages.getString("DatabaseSource.60") + (b - a) / 1000f //$NON-NLS-1$
					+ Messages.getString("DatabaseSource.61") + query + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
			((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.63") //$NON-NLS-1$
					+ (b - a) / 1000f);
			return tmp;
		} else {
			tmp = null;
			try {
				tmp = mysqldb.executeQuery(query);
				if (tmp != null)
					cache.put(query, tmp);
			} catch (OutOfMemoryError ome) {
				invalidateCache();
				JOptionPane
						.showMessageDialog(
								null,
								Messages.getString("DatabaseSource.64") //$NON-NLS-1$
										+ Messages.getString("DatabaseSource.65") //$NON-NLS-1$
										+ Messages.getString("DatabaseSource.66"), Messages.getString("MainFrame.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
				((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.68")); //$NON-NLS-1$
				logger
						.severe(Messages.getString("DatabaseSource.69")); //$NON-NLS-1$
				return tmp;
			}
			long b = System.currentTimeMillis();
			if (tmp != null) {
				logger.info(Messages.getString("DatabaseSource.70") + (float) (b - a) / 1000f //$NON-NLS-1$
						+ Messages.getString("DatabaseSource.61") + query + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
				((MainFrame) parent).setLogText(Messages.getString("DatabaseSource.70") //$NON-NLS-1$
						+ (float) (b - a) / 1000.000f);
			} else {
				logger.severe(Messages.getString("DatabaseSource.74") //$NON-NLS-1$
						+ query + "\n"); //$NON-NLS-1$
				((MainFrame) parent)
						.setLogText(Messages.getString("DatabaseSource.76")); //$NON-NLS-1$

			}
			return tmp;
		}

	}

}

