/*
 * This file is part of MySQLConnectus by Sergio Decherchi 
 * 
 * Copyright (C) 2004 Sergio Decherchi 
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details (gpl.txt). You should have
 * received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 */

package mysqlconnectus;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import bsh.*;
import tools.*;
import dbconnectivity.*;

/** Main window of the program */
public class MainFrame extends JFrame {

	//General variables

	/** Interface to the DBMS */
	public DatabaseConnector currentAccess;

	/** OrederedData variable (see the class) */
	final protected OrderedData orderedData;

	/** Common state variables */
	protected String temp, currentQuery = null;

	/** Current selected DB */
	protected String currentDB = ""; //$NON-NLS-1$

	/** Tree flag */
	public boolean treeFlag = false;

	/** Main split */
	public JSplitPane mainSplit = new JSplitPane();

	/** Desktop for text/table frames */
	public JDesktopPane frameDesktop = new JDesktopPane();

	//Theme variables
	/** String rapresenting window theme */
	final static private String windowsStyle = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"; //$NON-NLS-1$

	/** String rapresenting java theme */
	final static protected String javaStyle = "javax.swing.plaf.metal.MetalLookAndFeel"; //$NON-NLS-1$

	/** String rapresenting motif theme */
	final static private String motifStyle = "com.sun.java.swing.plaf.motif.MotifLookAndFeel"; //$NON-NLS-1$

	/** String rapresenting GTK theme */
	final static private String GTKStyle = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel"; //$NON-NLS-1$

	//Main window variables
	/** Internal frames title */
	protected String windowTitle = ""; //$NON-NLS-1$

	/** The reference for internal frames */
	public JDesktopPane desktop = null;

	/** Help on line window */
	private HtmlHelpOnLine helpOnLine = null;

	/** This reference variable rapresents the current instances of the tables */
	protected Table tab, savedTable = null;

	/**
	 * Index for the location of the query windows when a new window is opened
	 * this is placed in a different position respect the previuos one
	 */
	protected int winpos = 0;

	/** Instance for the window quering password/user */
	private PasswordChooser dialog = null;

	/** Current tree */
	public Tree currentTree = null;

	/** Flag for showing tables in Tree */
	public boolean showImmediatelyRelations = false;

	/** Flag for dialog windows management */
	public boolean accessDone = false;

	/** Pointer to the current MainFrame instance */
	public MainFrame currentMainFrame = this;

	/** Settings manager */
	protected SettingsManager settings;

	/** Import table instance */
	protected ImportTable it;

	/** Log window */
	public LogWindow currentLog;

	/** Resource bundle for localization */
	private ResourceBundle Messages = ResourceBundle.getBundle("Lang/messages"); //$NON-NLS-1$
	
	/** Plugins path */
	protected String pluginPath = ((((System.getProperties().getProperty("user.dir")).toString())+"/Plugins/").replace('\\', '/')); //$NON-NLS-1$
	
	/**Last selected directory */
	public File lastAccessedDirectory=new File(".");

	/** Main constructor */
	public MainFrame() {

		/*
		 * UIManager.LookAndFeelInfo[]
		 * themes=UIManager.getInstalledLookAndFeels(); for(int i=0;i
		 * <themes.length;i++) System.out.println(themes[i]);
		 */

		/* Record start time */
		
		long start = System.currentTimeMillis();
		settings = new SettingsManager(this);
		//	Save oldlog.txt file
		FileManager fm = new FileManager();
		fm.copy("log.txt", "oldlog.txt", false);
		it = new ImportTable(this);
		//MySQL Database connection Instance
		//Splash screen
		JWindow splashScreen = null;

		JLabel splashLabel = null;

		boolean noSplash = false;
		//Screen information
		int screenWidth = 0;
		int screenHeight = 0;
		//Tools bar
		JToolBar jtb = null;
		JProgressBar p = new JProgressBar(0, 100);
		p.setStringPainted(true);

		try {
			splashLabel = new JLabel(new ImageIcon(getClass().getResource(
					"/Resources/Splash2.jpg"))); //$NON-NLS-1$
			splashScreen = new JWindow(MainFrame.this);
			Panel pa = new Panel(new BorderLayout());
			pa.add(p, BorderLayout.SOUTH);
			pa.add(splashLabel, BorderLayout.CENTER);

			splashScreen.getContentPane().add(pa);

			splashScreen.pack();
			Rectangle screenRect = (MainFrame.this).getGraphicsConfiguration()
					.getBounds();

			splashScreen
					.setLocation(screenRect.x + screenRect.width / 2
							- splashScreen.getSize().width / 2, screenRect.y
							+ screenRect.height / 2
							- splashScreen.getSize().height / 2);

			splashScreen.pack();
			splashScreen.setVisible(true);

		} catch (NullPointerException npe) {
			JOptionPane.showMessageDialog(null, Messages
					.getString("MainFrame.nosplash"), //$NON-NLS-1$
					Messages.getString("MainFrame.Error"), 0); //$NON-NLS-1$
			;
			noSplash = true;
		}
		orderedData = new OrderedData();
		//Setup main frame
		setTitle("MySQLConnectus 0.4.0"); //$NON-NLS-1$

		Toolkit kit = Toolkit.getDefaultToolkit();
		Dimension screenSize = kit.getScreenSize();
		screenWidth = ((screenSize.width));
		screenHeight = ((screenSize.height) - 25);
		setLocation(0, 0);
		setSize(screenWidth, screenHeight);
		//Creation of the main window
		MainPanel panel = new MainPanel();
		desktop = new JDesktopPane();
		currentLog = new LogWindow(null, currentMainFrame);
		desktop.add(currentLog);
		Container contentPane = getContentPane();
		jtb = new JToolBar();
		p.setValue(33);
		//Setting toolbar
		JButton writeqb = new JButton(new ImageIcon(
				"Resources/palette_draw_query.gif")); //$NON-NLS-1$
		JButton openb = new JButton(new ImageIcon("Resources/openquery.gif")); //$NON-NLS-1$
		JButton writemb = new JButton(new ImageIcon(
				"Resources/palette_draw_mod.gif")); //$NON-NLS-1$
		JButton openmb = new JButton(new ImageIcon("Resources/openmod.gif")); //$NON-NLS-1$
		JButton loginb = new JButton(new ImageIcon("Resources/next.gif")); //$NON-NLS-1$
		JButton scriptb = new JButton(new ImageIcon("Resources/script.gif")); //$NON-NLS-1$
		JButton opentb = new JButton(new ImageIcon("Resources/opentab.gif")); //$NON-NLS-1$
		JButton showtb = new JButton(new ImageIcon(
				"Resources/showrelations.gif")); //$NON-NLS-1$
		JButton chdbb = new JButton(new ImageIcon("Resources/change.gif")); //$NON-NLS-1$
		JButton showb = new JButton(
				new ImageIcon("Resources/showdatabases.gif")); //$NON-NLS-1$
		JButton autob = new JButton(new ImageIcon("Resources/gear.gif")); //$NON-NLS-1$
		JButton helpb = new JButton(new ImageIcon("Resources/HelpCenter.gif")); //$NON-NLS-1$
		JButton exitb = new JButton(new ImageIcon("Resources/prev.gif")); //$NON-NLS-1$
		openb.addActionListener(new LoadQuery());
		chdbb.addActionListener(new Chdb());
		showb.addActionListener(new Show());
		writeqb.addActionListener(new WriteQuery());
		writemb.addActionListener(new WriteModification());
		openmb.addActionListener(new Mod());
		loginb.addActionListener(new ConnectAction());
		scriptb.addActionListener(new Script());
		opentb.addActionListener(new OpenSaved());
		helpb.addActionListener(new Help());
		autob.addActionListener(new Config());
		showtb.addActionListener(new Showtb());
		p.setValue(36);
		exitb.addActionListener(new Exit());
		openb.setToolTipText(Messages.getString("MainFrame.open_query")); //$NON-NLS-1$
		chdbb.setToolTipText(Messages.getString("MainFrame.chdb")); //$NON-NLS-1$
		showtb.setToolTipText(Messages.getString("MainFrame.show_relations")); //$NON-NLS-1$
		showb.setToolTipText(Messages.getString("MainFrame.show_databases")); //$NON-NLS-1$
		writeqb.setToolTipText(Messages.getString("MainFrame.writeaquery")); //$NON-NLS-1$
		writemb.setToolTipText(Messages.getString("MainFrame.writeamod")); //$NON-NLS-1$
		openmb.setToolTipText(Messages.getString("MainFrame.openamod")); //$NON-NLS-1$
		loginb.setToolTipText(Messages.getString("MainFrame.30")); //$NON-NLS-1$
		scriptb.setToolTipText(Messages.getString("MainFrame.loadexscript")); //$NON-NLS-1$
		opentb.setToolTipText(Messages.getString("MainFrame.opentab")); //$NON-NLS-1$
		helpb.setToolTipText(Messages.getString("MainFrame.helponline")); //$NON-NLS-1$
		autob.setToolTipText(Messages.getString("MainFrame.autoconf")); //$NON-NLS-1$
		exitb.setToolTipText(Messages.getString("MainFrame.exit")); //$NON-NLS-1$
		jtb.add(loginb);
		jtb.add(scriptb);
		jtb.addSeparator();
		jtb.add(showb);
		jtb.add(chdbb);
		jtb.add(showtb);
		jtb.addSeparator();
		jtb.addSeparator();
		jtb.add(writeqb);
		jtb.add(openb);
		jtb.addSeparator();
		jtb.add(writemb);
		jtb.add(openmb);
		jtb.addSeparator();
		jtb.add(opentb);
		jtb.addSeparator();
		jtb.add(autob);
		jtb.add(helpb);
		jtb.addSeparator();
		jtb.add(exitb);
		p.setValue(40);
		contentPane.add(panel);
		contentPane.add(jtb, BorderLayout.NORTH);
		contentPane.add(desktop);
		//Creation of the menu and submenus
		JMenuBar menubar = new JMenuBar();
		JMenu help = new JMenu(Messages.getString("MainFrame.help")); //$NON-NLS-1$
		JMenu look = new JMenu("LookAndFeel!"); //$NON-NLS-1$
		JMenu java = new JMenu(Messages.getString("MainFrame.Java_Styles")); //$NON-NLS-1$
		JMenuItem windows = new JMenuItem(Messages
				.getString("MainFrame.Windows_Style")); //$NON-NLS-1$
		JMenuItem GTK = new JMenuItem(Messages.getString("MainFrame.GTK_Style")); //$NON-NLS-1$
		JMenuItem def = new JMenuItem(Messages
				.getString("MainFrame.Java_theme")); //$NON-NLS-1$
		JMenuItem motif = new JMenuItem(Messages
				.getString("MainFrame.Motif_Style")); //$NON-NLS-1$
		final JMenuItem about = new JMenuItem(Messages
				.getString("MainFrame.43"), new ImageIcon( //$NON-NLS-1$
				"Resources/tip.gif")); //$NON-NLS-1$
		JMenuItem ruby = new JMenuItem(Messages.getString("MainFrame.rth")); //$NON-NLS-1$
		JMenuItem aqua = new JMenuItem(Messages.getString("MainFrame.aqt")); //$NON-NLS-1$
		JMenuItem emerald = new JMenuItem(Messages.getString("MainFrame.emt")); //$NON-NLS-1$
		JMenuItem charcoal = new JMenuItem(Messages.getString("MainFrame.cht")); //$NON-NLS-1$
		JMenuItem contrast = new JMenuItem(Messages.getString("MainFrame.cot")); //$NON-NLS-1$
		JMenu fileMenu = new JMenu("File"); //$NON-NLS-1$
		JMenu options = new JMenu(Messages.getString("MainFrame.opt")); //$NON-NLS-1$
		JMenuItem config = new JMenuItem(Messages
				.getString("MainFrame.autconf"), new ImageIcon( //$NON-NLS-1$
				"Resources/gear.gif")); //$NON-NLS-1$
		JMenuItem show = new JMenuItem(Messages
				.getString("MainFrame.show_databases"), new ImageIcon( //$NON-NLS-1$
				"Resources/showdatabases.gif")); //$NON-NLS-1$
		JMenuItem showTb = new JMenuItem(Messages
				.getString("MainFrame.show_relations"), new ImageIcon( //$NON-NLS-1$
				"Resources/showrelations.gif")); //$NON-NLS-1$
		JMenuItem loadQuery = new JMenuItem(Messages
				.getString("MainFrame.openquery"), new ImageIcon( //$NON-NLS-1$
				"Resources/openquery.gif")); //$NON-NLS-1$
		JMenuItem openSaved = new JMenuItem(Messages
				.getString("MainFrame.opentab"), new ImageIcon( //$NON-NLS-1$
				"Resources/opentab.gif")); //$NON-NLS-1$
		JMenuItem importTableFromTXT = new JMenuItem(Messages
				.getString("MainFrame.import_into_exisiting"), new ImageIcon( //$NON-NLS-1$
				"Resources/importab.gif")); //$NON-NLS-1$
		JMenuItem importIntoNewTableFromTXT = new JMenuItem(Messages
				.getString("MainFrame.create&import"), //$NON-NLS-1$
				new ImageIcon("Resources/importab.gif")); //$NON-NLS-1$
		JMenuItem mod = new JMenuItem(Messages
				.getString("MainFrame.openupdein"), new ImageIcon( //$NON-NLS-1$
				"Resources/openmod.gif")); //$NON-NLS-1$
		JMenuItem login = new JMenuItem("Log in", new ImageIcon( //$NON-NLS-1$
				"Resources/next.gif")); //$NON-NLS-1$
		JMenuItem logout = new JMenuItem("Log out", new ImageIcon( //$NON-NLS-1$
				"Resources/prev.gif")); //$NON-NLS-1$
		JMenuItem chdb = new JMenuItem(
				Messages.getString("MainFrame.chdb"), new ImageIcon( //$NON-NLS-1$
						"Resources/change.gif")); //$NON-NLS-1$
		JMenuItem exit = new JMenuItem(
				Messages.getString("MainFrame.exit"), new ImageIcon( //$NON-NLS-1$
						"Resources/prev.gif")); //$NON-NLS-1$
		JMenuItem write = new JMenuItem(Messages
				.getString("MainFrame.newquery"), new ImageIcon( //$NON-NLS-1$
				"Resources/palette_draw_query.gif")); //$NON-NLS-1$
		JMenuItem writeModification = new JMenuItem(Messages
				.getString("MainFrame.newupdein"), //$NON-NLS-1$
				new ImageIcon("Resources/palette_draw_mod.gif")); //$NON-NLS-1$
		JMenuItem script = new JMenuItem(Messages
				.getString("MainFrame.loadexscript"), //$NON-NLS-1$
				new ImageIcon("Resources/script.gif")); //$NON-NLS-1$
		JMenuItem helpOnLine = new JMenuItem(Messages
				.getString("MainFrame.helponline"), new ImageIcon( //$NON-NLS-1$
				"Resources/HelpCenter.gif")); //$NON-NLS-1$
		JMenu plugin = new JMenu("Plugins"); //$NON-NLS-1$
		//System.out.println(pluginPath);
		File mypath = new File(pluginPath);
		File[] pluginList = mypath.listFiles();
		JMenuItem[] pluginItems = new JMenuItem[pluginList.length];
		//System.out.println(path + "\n" + pluginList);

		if (pluginList != null) {
			for (int i = 0; i < pluginList.length; i++) {
				pluginItems[i] = new JMenuItem(pluginList[i].getName());
				plugin.add(pluginItems[i]);
				pluginItems[i].addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent evt) {

						Interpreter bsh = new Interpreter();

						try {
							//bsh.set("currentMainFrame", this);
							FileManager fm = new FileManager();
							fm.readFile(pluginPath
									+ ((JMenuItem) evt.getSource()).getText());
							//System.out.println(((JMenuItem)evt.getSource()).getText());
							bsh.set("currentMainFrame", currentMainFrame);
							bsh.eval(fm.text);

						} catch (EvalError e1) {
							JOptionPane.showMessageDialog(null,
									e1.getMessage(), Messages
											.getString("MainFrame.W"), 1); //$NON-NLS-1$ 

							e1.printStackTrace();
							e1.getCause();
						}

					}
				});

			}
		}
		p.setValue(50);
		help.setMnemonic('H');
		fileMenu.setMnemonic('F');
		options.setMnemonic('O');
		fileMenu.add(login);
		fileMenu.add(logout);
		fileMenu.addSeparator();
		fileMenu.add(write);
		fileMenu.add(writeModification);
		fileMenu.addSeparator();
		fileMenu.add(loadQuery);
		fileMenu.add(mod);
		fileMenu.add(openSaved);
		fileMenu.addSeparator();
		fileMenu.add(importTableFromTXT);
		fileMenu.add(importIntoNewTableFromTXT);
		fileMenu.addSeparator();
		fileMenu.add(script);
		fileMenu.addSeparator();
		fileMenu.add(chdb);
		fileMenu.add(show);
		fileMenu.add(showTb);
		fileMenu.addSeparator();
		fileMenu.addSeparator();
		fileMenu.add(exit);
		look.add(windows);
		look.add(GTK);
		look.add(motif);
		look.add(java);
		java.add(def);
		java.add(ruby);
		java.add(aqua);
		java.add(emerald);
		java.add(charcoal);
		java.add(contrast);
		options.add(look);
		options.add(config);
		help.add(helpOnLine);
		help.add(about);
		p.setValue(66);
		menubar.add(fileMenu);
		menubar.add(options);
		menubar.add(plugin);
		menubar.add(help);

		setJMenuBar(menubar);
		//Actions associated with menu voices
		loadQuery.addActionListener(new LoadQuery());
		openSaved.addActionListener(new OpenSaved());
		mod.addActionListener(new Mod());
		exit.addActionListener(new Exit());
		script.addActionListener(new Script());
		show.addActionListener(new Show());
		showTb.addActionListener(new Showtb());
		login.addActionListener(new ConnectAction());
		logout.addActionListener(new DisconnectAction());
		config.addActionListener(new Config());
		chdb.addActionListener(new Chdb());
		helpOnLine.addActionListener(new Help());
		write.addActionListener(new WriteQuery());
		writeModification.addActionListener(new WriteModification());
		importTableFromTXT.addActionListener(new ImportTableAction());
		importIntoNewTableFromTXT
				.addActionListener(new ImportCreateTableAction());
		p.setValue(80);
		about.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				JInternalFrame jf = new JInternalFrame(Messages
						.getString("MainFrame.about")); //$NON-NLS-1$
				jf.setFrameIcon(new ImageIcon("Resources/tip.gif"));
				jf.setResizable(false);
				jf.setLocation(450, 0);
				jf.setSize(490, 318);
				Properties prop = System.getProperties();
				Runtime runtimeStatus = Runtime.getRuntime();
				int free = (int) (runtimeStatus.freeMemory() / 1024);
				int total = (int) (runtimeStatus.totalMemory() / 1024);
				final JProgressBar progress = new JProgressBar(0, total);
				progress.setValue(free);
				progress.setStringPainted(true);
				progress.setString((new Integer(free)).toString()
						+ Messages.getString("MainFrame.KB_FREE") //$NON-NLS-1$
						+ (new Integer(total)).toString()
						+ Messages.getString("MainFrame.KB_TOTAL")); //$NON-NLS-1$
				JTextArea jta = new JTextArea();
				Panel p1 = new Panel(new BorderLayout());
				jta.setEditable(false);
				jta.append(Messages.getString("MainFrame.user") //$NON-NLS-1$
						+ currentAccess.getConnectionData().getUsername()
						+ Messages.getString("MainFrame.ip") //$NON-NLS-1$
						+ currentAccess.getConnectionData().getHost()
						+ Messages.getString("MainFrame.sysinfo") //$NON-NLS-1$
						+ "\nJVM version : " //$NON-NLS-1$
						+ prop.getProperty("java.vm.version") //$NON-NLS-1$
						+ "\nOS : " //$NON-NLS-1$
						+ prop.getProperty("os.name") //$NON-NLS-1$
						+ " " //$NON-NLS-1$
						+ prop.getProperty("os.version") //$NON-NLS-1$
						+ Messages.getString("MainFrame.#procs") //$NON-NLS-1$
						+ runtimeStatus.availableProcessors()
						+ Messages.getString("MainFrame.about_contact")); //$NON-NLS-1$
				//jta.append("Ciao\n\n\nCiao");
				JScrollPane jsp = new JScrollPane(jta);
				p1.add(jsp, BorderLayout.CENTER);
				Panel p3 = new Panel(new BorderLayout());
				p1.add(progress, BorderLayout.SOUTH);
				Panel p2 = new Panel(new BorderLayout());

				JButton gc = (new JButton(Messages.getString("MainFrame.gc"))); //$NON-NLS-1$
				JButton ic = (new JButton(Messages
						.getString("MainFrame.invalidate_cache"))); //$NON-NLS-1$

				p2.add(gc, BorderLayout.EAST);
				p2.add(ic, BorderLayout.WEST);
				p3.add(p1, BorderLayout.NORTH);
				p3.add(p2, BorderLayout.WEST);

				JTabbedPane jtp = new JTabbedPane();

				Panel p4 = new Panel(new BorderLayout());
				JTextArea jta2 = new JTextArea();
				FileManager fm = new FileManager();
				fm.readFile("log.txt"); //$NON-NLS-1$
				JScrollPane jsp2 = new JScrollPane(jta2);
				p4.add(jsp2, BorderLayout.CENTER);
				jta2.setText(fm.text);
				jta2.setCaretPosition(0);
				jta2.setEditable(false);

				Panel p5 = new Panel(new BorderLayout());
				JTextArea jta3 = new JTextArea();
				fm.readFile("oldlog.txt"); //$NON-NLS-1$
				JScrollPane jsp3 = new JScrollPane(jta3);
				p5.add(jsp3, BorderLayout.CENTER);
				jta3.setText(fm.text);
				jta3.setCaretPosition(0);
				jta3.setEditable(false);

				Panel p6 = new Panel(new BorderLayout());
				JTextArea jta4 = new JTextArea();
				JScrollPane jsp4 = new JScrollPane(jta4);
				p6.add(jsp4, BorderLayout.CENTER);
				jta4
						.setText("Copyright (C) 2004 Sergio Decherchi \n" + //$NON-NLS-1$
								"This program is free software; you can redistribute it \n"
								+ //$NON-NLS-1$
								"and/or modify it under the terms of the \nGNU General Public License as \n"
								+ //$NON-NLS-1$
								"published by the Free Software Foundation;\neither version 2 of the License, \n"
								+ //$NON-NLS-1$
								"or (at your option) any later version. \nThis program is distributed in the\n"
								+ //$NON-NLS-1$
								"hope that it will be useful, but WITHOUT ANY WARRANTY;\nwithout even the "
								+ //$NON-NLS-1$
								"implied warranty of MERCHANTABILITY \nor FITNESS FOR A PARTICULAR PURPOSE.\nSee "
								+ //$NON-NLS-1$
								" the GNU General Public License \nfor more details (gpl.txt). \nYou should have "
								+ //$NON-NLS-1$
								"received a copy of the \nGNU General Public License along with this program;\nif "
								+ //$NON-NLS-1$
								"not, write to the Free Software Foundation, Inc., \n59 Temple Place, Suite 330, \n"
								+ //$NON-NLS-1$
								"Boston, MA 02111-1307 USA\n"); //$NON-NLS-1$
				jta4.setCaretPosition(0);
				jta4.setEditable(false);

				jtp.add(p3);
				jtp.add(p4);
				jtp.add(p5);
				jtp.add(p6);

				jtp.setTitleAt(0, Messages.getString("MainFrame.geninfo")); //$NON-NLS-1$
				jtp.setTitleAt(1, Messages.getString("MainFrame.curlog")); //$NON-NLS-1$
				jtp.setTitleAt(2, Messages.getString("MainFrame.prevlog")); //$NON-NLS-1$
				jtp.setTitleAt(3, Messages.getString("MainFrame.license")); //$NON-NLS-1$

				jf.add(jtp);
				jf.setClosable(true);
				desktop.add(jf);
				jf.invalidate();
				jf.validate();
				jf.repaint();
				jf.setVisible(true);

				gc.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent evt) {
						System.gc();
						int free = (int) (Runtime.getRuntime().freeMemory() / 1024);
						int total = (int) (Runtime.getRuntime().totalMemory() / 1024);
						progress.setValue(free);
						progress.setStringPainted(true);
						progress
								.setString((new Integer(free)).toString()
										+ Messages
												.getString("MainFrame.KB_FREE") + (new Integer(total)).toString() //$NON-NLS-1$
										+ Messages
												.getString("MainFrame.KB_TOTAL")); //$NON-NLS-1$

					}
				});

				ic.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent evt) {
						((DatabaseSource) currentAccess).invalidateCache();
						int free = (int) (Runtime.getRuntime().freeMemory() / 1024);
						int total = (int) (Runtime.getRuntime().totalMemory() / 1024);
						progress.setValue(free);
						progress.setStringPainted(true);
						progress
								.setString((new Integer(free)).toString()
										+ Messages
												.getString("MainFrame.KB_FREE") + (new Integer(total)).toString() //$NON-NLS-1$
										+ Messages
												.getString("MainFrame.KB_TOTAL")); //$NON-NLS-1$

					}
				});

			}
		});
		windows.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(false, 0, windowsStyle);
				settings.nextSessionIsJavaStyle = false;
				settings.nextSessionJavaTheme = 0;
				settings.nextSessionStyle = windowsStyle;
			}
		});
		GTK.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(false, 0, GTKStyle);
				settings.nextSessionIsJavaStyle = false;
				settings.nextSessionJavaTheme = 0;
				settings.nextSessionStyle = GTKStyle;
			}
		});
		p.setValue(89);
		def.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 1, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 1;
				settings.nextSessionStyle = javaStyle;
			}
		});
		motif.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(false, 0, motifStyle);
				settings.nextSessionIsJavaStyle = false;
				settings.nextSessionJavaTheme = 0;
				settings.nextSessionStyle = motifStyle;
			}
		});
		ruby.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 2, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 2;
				settings.nextSessionStyle = javaStyle;
			}
		});
		aqua.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 6, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 6;
				settings.nextSessionStyle = javaStyle;
			}
		});
		charcoal.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 4, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 4;
				settings.nextSessionStyle = javaStyle;
			}
		});
		contrast.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 5, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 5;
				settings.nextSessionStyle = javaStyle;
			}
		});
		emerald.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				settings.setNewTheme(true, 3, javaStyle);
				settings.nextSessionIsJavaStyle = true;
				settings.nextSessionJavaTheme = 3;
				settings.nextSessionStyle = javaStyle;
			}
		});
		p.setValue(91);
		this.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				settings.saveSettings();
				System.exit(0);
			}
		});

		long finish = System.currentTimeMillis();
		currentLog
				.addText(Messages.getString("MainFrame.start_message") + (finish - start) //$NON-NLS-1$
						/ 1000);
		currentAccess = new DatabaseSource(DatabaseSource.MYSQL,
				currentMainFrame);
		frameDesktop.setBackground(new Color(220, 220, 255));
		JSplitPane logSplit = new JSplitPane();
		mainSplit.setLocation(0, 0);
		mainSplit.setDividerLocation(200);
		mainSplit.setVisible(true);

		//Set splitting panels
		mainSplit.setRightComponent(frameDesktop);
		//mainSplit.setResizeWeight(0.15);

		logSplit.setLocation(0, screenHeight - 270);
		logSplit.setOrientation(0);
		logSplit.setDividerLocation(300);
		logSplit.setLeftComponent(currentLog);
		logSplit.setSize(screenWidth - 7, 190);
		logSplit.setVisible(true);
		mainSplit.setSize(screenWidth, screenHeight - 270);
		desktop.add(logSplit);
		desktop.add(mainSplit);
		TextEditFrame newtef = new TextEditFrame(this, true, Messages
				.getString("TextEditFrame.1"));

		//		Set or not autoconfig settings
		settings.autoconfig();
		System.gc();
		p.setValue(100);
		splashScreen.setVisible(false);
		//System.out.println(nextSessionJavaTheme);

	}

	public void setLogText(String msg) {
		currentLog.addText(msg);
	}

	/** Demo method */
	public void robot() {
		try {

			Robot rb = new Robot();
			JOptionPane.showMessageDialog(null, Messages
					.getString("MainFrame.demosession") //$NON-NLS-1$
					+ Messages.getString("MainFrame.demorules") //$NON-NLS-1$
					+ Messages.getString("MainFrame.demostart"), "DEMO", 1); //$NON-NLS-1$ //$NON-NLS-2$
			/* Opening connection */
			JOptionPane.showMessageDialog(null, Messages
					.getString("MainFrame.demolog"), //$NON-NLS-1$
					"DEMO", 1); //$NON-NLS-1$

			rb.keyPress(KeyEvent.VK_ALT);
			rb.keyPress(KeyEvent.VK_F);
			rb.delay(700);
			rb.keyRelease(KeyEvent.VK_F);
			rb.keyRelease(KeyEvent.VK_ALT);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(2000);
			rb.keyPress(KeyEvent.VK_R);
			rb.keyRelease(KeyEvent.VK_R);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_O);
			rb.keyRelease(KeyEvent.VK_O);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_O);
			rb.keyRelease(KeyEvent.VK_O);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_T);
			rb.keyRelease(KeyEvent.VK_T);
			rb.keyPress(KeyEvent.VK_TAB);
			rb.keyRelease(KeyEvent.VK_TAB);
			rb.keyPress(KeyEvent.VK_TAB);
			rb.keyRelease(KeyEvent.VK_TAB);
			rb.keyPress(KeyEvent.VK_TAB);
			rb.keyRelease(KeyEvent.VK_TAB);
			rb.delay(3000);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(3000);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			/* Entering mysql db */
			rb.delay(1000);
			rb.keyPress(KeyEvent.VK_M);
			rb.keyRelease(KeyEvent.VK_M);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_Y);
			rb.keyRelease(KeyEvent.VK_Y);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_S);
			rb.keyRelease(KeyEvent.VK_S);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_Q);
			rb.keyRelease(KeyEvent.VK_Q);
			rb.delay(500);
			rb.keyPress(KeyEvent.VK_L);
			rb.keyRelease(KeyEvent.VK_L);
			rb.delay(3000);

			rb.keyPress(KeyEvent.VK_ENTER);
			rb.delay(400);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.delay(400);
			rb.keyRelease(KeyEvent.VK_ENTER);
			JOptionPane.showMessageDialog(null, Messages
					.getString("MainFrame.demotree"), //$NON-NLS-1$
					"DEMO", 1); //$NON-NLS-1$

			rb.delay(1000);
			/* Opening query window */
			rb.keyPress(KeyEvent.VK_ALT);
			rb.keyPress(KeyEvent.VK_F);
			rb.delay(700);
			rb.keyRelease(KeyEvent.VK_F);
			rb.keyRelease(KeyEvent.VK_ALT);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.delay(400);
			rb.keyRelease(KeyEvent.VK_ENTER);
			/* Doing a query */
			rb.keyPress(KeyEvent.VK_A);
			rb.delay(400);
			rb.keyRelease(KeyEvent.VK_A);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(300);
			rb.mouseMove(300, 300);
			rb.mousePress(InputEvent.BUTTON1_MASK);
			rb.mouseRelease(InputEvent.BUTTON1_MASK);

			rb.keyPress(KeyEvent.VK_ALT);
			rb.keyPress(KeyEvent.VK_A);
			rb.delay(400);
			rb.keyRelease(KeyEvent.VK_A);
			rb.keyRelease(KeyEvent.VK_ALT);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(400);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(2000);
			/* Writing query */
			rb.keyPress(KeyEvent.VK_S);
			rb.keyRelease(KeyEvent.VK_S);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_E);
			rb.keyRelease(KeyEvent.VK_E);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_L);
			rb.keyRelease(KeyEvent.VK_L);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_E);
			rb.keyRelease(KeyEvent.VK_E);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_C);
			rb.keyRelease(KeyEvent.VK_C);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_T);
			rb.keyRelease(KeyEvent.VK_T);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_SPACE);
			rb.keyRelease(KeyEvent.VK_SPACE);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_MULTIPLY);
			rb.keyRelease(KeyEvent.VK_MULTIPLY);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_SPACE);
			rb.keyRelease(KeyEvent.VK_SPACE);
			rb.delay(200);

			rb.keyPress(KeyEvent.VK_F);
			rb.keyRelease(KeyEvent.VK_F);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_R);
			rb.keyRelease(KeyEvent.VK_R);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_O);
			rb.keyRelease(KeyEvent.VK_O);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_M);
			rb.keyRelease(KeyEvent.VK_M);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_SPACE);
			rb.keyRelease(KeyEvent.VK_SPACE);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_U);
			rb.keyRelease(KeyEvent.VK_U);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_S);
			rb.keyRelease(KeyEvent.VK_S);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_E);
			rb.keyRelease(KeyEvent.VK_E);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_R);
			rb.keyRelease(KeyEvent.VK_R);
			rb.delay(200);
			rb.keyPress(KeyEvent.VK_ALT);
			rb.keyPress(KeyEvent.VK_A);
			rb.delay(700);
			rb.keyRelease(KeyEvent.VK_A);
			rb.keyRelease(KeyEvent.VK_ALT);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_ALT);
			rb.keyPress(KeyEvent.VK_A);
			rb.delay(700);
			rb.keyRelease(KeyEvent.VK_A);
			rb.keyRelease(KeyEvent.VK_ALT);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_DOWN);
			rb.keyRelease(KeyEvent.VK_DOWN);
			rb.delay(700);
			rb.keyPress(KeyEvent.VK_ENTER);
			rb.keyRelease(KeyEvent.VK_ENTER);
			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("MainFrame.demoended"), Messages.getString("MainFrame.demo"), 1); //$NON-NLS-1$ //$NON-NLS-2$

		} catch (Exception awte) {
		}

	}

	/** Set current db internal rapresentation */
	public void setCurrentDB(String db) {

		currentDB = db;
	}

	//The different action listeners used in the program.
	/** Create an empty window in which you can insert your query text */
	private class WriteQuery implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			windowTitle = Messages.getString("MainFrame.querytext"); //$NON-NLS-1$
			(new TextEditFrame(currentMainFrame, true, Messages
					.getString("TextEditFrame.1"))).show();
			System.gc();
		}
	}

	/** Create an empty window in which you can insert your modification text */
	private class WriteModification implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			windowTitle = Messages.getString("MainFrame.modtext"); //$NON-NLS-1$
			(new TextEditFrame(currentMainFrame, false, Messages
					.getString("TextEditFrame.3"))).show();
			System.gc();
		}
	}

	/** Open help on line window */
	private class Help implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (helpOnLine != null) {
				helpOnLine.doDefaultCloseAction();
			}
			helpOnLine = new HtmlHelpOnLine(frameDesktop);
			System.gc();
		}
	}

	/** Change db class */
	private class Chdb implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (accessDone) {
				String oldDB;
				oldDB = new String(currentDB.toString());
				currentDB = null;
				currentDB = (JOptionPane.showInputDialog(null, Messages
						.getString("MainFrame.Insert_db_name"), "Info", 1)); //$NON-NLS-1$ //$NON-NLS-2$
				if (currentDB == null) {
					currentDB = oldDB;
					return;
				}
				if ((currentDB == null)
						|| (currentAccess.executeUpdein(("USE ") //$NON-NLS-1$
								.concat(currentDB), true) == false)) {
					if (currentTree != null) {
						currentTree.doDefaultCloseAction();
					}
					if (currentDB != null) {
						JOptionPane.showMessageDialog(null, Messages
								.getString("MainFrame.db") //$NON-NLS-1$
								+ currentDB
								+ Messages.getString("MainFrame.notpresent"), //$NON-NLS-1$
								Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
						currentDB = oldDB;
					} else
						currentDB = oldDB;
					showImmediatelyRelations = true;
					currentTree = new Tree(currentMainFrame);
					currentTree.updateTree();
				} else {
					showImmediatelyRelations = true;
					if (currentTree != null) {
						currentTree.doDefaultCloseAction();
					}
					currentTree = new Tree(currentMainFrame);
					currentTree.updateTree();

				}
			} else
				JOptionPane.showMessageDialog(null, Messages
						.getString("MainFrame.nolog"), //$NON-NLS-1$
						Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
			System.gc();
		}
	}

	/** Panel for option settings */
	private class Config implements ActionListener {

		public void actionPerformed(ActionEvent e) {

			new OptionPanel(currentMainFrame, desktop);
		}
	}

	/** Load a query by a saved one */
	private class LoadQuery implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			JFileChooser chooser = new JFileChooser();
			chooser.setCurrentDirectory(lastAccessedDirectory); //$NON-NLS-1$
			chooser.addChoosableFileFilter(new QueryFilter());
			chooser.setFileView(new FileIconViewForQuery(new QueryFilter(),
					new ImageIcon("Resources/query.gif"))); //$NON-NLS-1$
			int r = chooser.showOpenDialog(null);
			
			if (r == JFileChooser.APPROVE_OPTION) {
				lastAccessedDirectory=chooser.getSelectedFile();
				windowTitle = chooser.getSelectedFile().getName();
				temp = chooser.getSelectedFile().getAbsolutePath();
				TextEditFrame tef;
				FileManager fm = new FileManager();
				if (!fm.readFile(temp))
					tef = new TextEditFrame(currentMainFrame, true, fm.text);
			}
			System.gc();
		}
	}

	/** Open a saved table */
	private class OpenSaved implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			JFileChooser chooser = new JFileChooser();
			chooser.setCurrentDirectory(lastAccessedDirectory); //$NON-NLS-1$
			chooser.addChoosableFileFilter(new TabFilter());
			chooser.setFileView(new FileIconViewForTab(new TabFilter(),
					new ImageIcon("Resources/JTable.gif"))); //$NON-NLS-1$
			int r = chooser.showOpenDialog(null);
			if (r == JFileChooser.APPROVE_OPTION) {
				lastAccessedDirectory=chooser.getSelectedFile();
				windowTitle = chooser.getSelectedFile().getName();
				temp = chooser.getSelectedFile().getAbsolutePath();
				try {
					FileInputStream fis = new FileInputStream(temp);
					ObjectInputStream ois = new ObjectInputStream(fis);
					BufferedResultSet prs = new BufferedResultSet(temp, ois);
					savedTable = new Table(prs.doc, prs.columnNames,
							frameDesktop, windowTitle, prs.columnType,
							currentMainFrame);
				} catch (FileNotFoundException fnfe) {
					JOptionPane
							.showMessageDialog(
									null,
									Messages.getString("MainFrame.file") + temp //$NON-NLS-1$
											+ Messages
													.getString("MainFrame.notpresent"), Messages.getString("MainFrame.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
				} catch (IOException ioe) {
					JOptionPane.showMessageDialog(null, Messages
							.getString("MainFrame.IOError"), "Error", 0); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			System.gc();
		}
	}

	/** Import a table from a txt file */
	private class ImportTableAction implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			it.importFromTxt();
		}

	}

	/**
	 * Import from filename.txt and filename@metadata.txt to create and populate
	 * a table.
	 */
	private class ImportCreateTableAction implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			it.importCreateFromTxt();
		}

	}

	/** Load a modification file */
	private class Mod implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			JFileChooser chooser = new JFileChooser();
			chooser.setCurrentDirectory(lastAccessedDirectory); //$NON-NLS-1$
			chooser.addChoosableFileFilter(new ModFilter());
			chooser.setFileView(new FileIconViewForMod(new ModFilter(),
					new ImageIcon("Resources/mod.gif"))); //$NON-NLS-1$
			int r = chooser.showOpenDialog(null);
			if (r == JFileChooser.APPROVE_OPTION) {
				lastAccessedDirectory= chooser.getSelectedFile();
				windowTitle = chooser.getSelectedFile().getName();
				temp = chooser.getSelectedFile().getAbsolutePath();
				FileManager fm = new FileManager();
				if (!fm.readFile(temp))
					(new TextEditFrame(currentMainFrame, false, fm.text))
							.show();
			}
			System.gc();
		}
	}

	/** Save settings and exit */
	private class Exit implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			settings.saveSettings();
			System.exit(0);
		}
	}

	/** Load and execute a script file */
	private class Script implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			JFileChooser chooser = new JFileChooser();
			chooser.setCurrentDirectory(lastAccessedDirectory); //$NON-NLS-1$
			int r = chooser.showOpenDialog(null);
			if (r == JFileChooser.APPROVE_OPTION) {
				lastAccessedDirectory=chooser.getSelectedFile();
				temp = chooser.getSelectedFile().getAbsolutePath();
				if (currentTree != null) {
					currentTree.doDefaultCloseAction();
				}
				currentTree = new Tree(currentMainFrame);

				currentAccess.executeScript(temp);
				if (!settings.scriptSet) {
					settings.nextSessionScript = temp;
				} else {
					Object[] options = {
							Messages.getString("MainFrame.yes"), Messages.getString("MainFrame.no")}; //$NON-NLS-1$ //$NON-NLS-2$
					int choosen = JOptionPane
							.showOptionDialog(
									null,
									Messages.getString("MainFrame.rescript"), //$NON-NLS-1$
									Messages.getString("MainFrame.Script"), JOptionPane.DEFAULT_OPTION, //$NON-NLS-1$
									JOptionPane.QUESTION_MESSAGE, null,
									options, options[0]);
					if (choosen == 0) {
						settings.nextSessionScript = temp;
						settings.scriptSet = false;
					}
				}
			}
			System.gc();
		}
	}

	/** Show databases in the current tree */
	private class Show implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (currentTree != null) {
				currentTree.doDefaultCloseAction();
			}
			currentTree = new Tree(currentMainFrame);
			currentTree.updateTree();
		}
	}

	/** Show relations in the current tree */
	private class Showtb implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (currentTree != null) {
				currentTree.doDefaultCloseAction();
			}
			currentTree = new Tree(currentMainFrame);
			showImmediatelyRelations = true;
			currentTree.updateTree();
		}
	}

	/** Class that manages the connection step */
	private class ConnectAction implements ActionListener {

		public void actionPerformed(ActionEvent event) {

			String tmp1, tmp2, tmp3;

			if (dialog == null)
				dialog = new PasswordChooser(currentMainFrame);
			dialog.setUser(new User(null, null));
			if (dialog.showDialog(MainFrame.this, Messages
					.getString("MainFrame.connect"))) { //$NON-NLS-1$
				tmp1 = (dialog.getUser()).getName();
				tmp2 = (dialog.getUser()).getPassword();
				tmp3 = (dialog.getIP());
				if (tmp3.equals("")) //$NON-NLS-1$
					tmp3 = "127.0.0.1"; //$NON-NLS-1$
				String ex = currentAccess.access(tmp1, tmp2, tmp3);
				JOptionPane.showMessageDialog(null, ex, "Info", 1); //$NON-NLS-1$
				if (ex.equals(Messages.getString("Axxess.GoodConnection"))) { //$NON-NLS-1$
					if (currentTree != null) {
						currentTree.doDefaultCloseAction();
					}
					currentTree = new Tree(currentMainFrame);

					accessDone = true;
					currentDB = (JOptionPane.showInputDialog(null, Messages
							.getString("MainFrame.Insert_db_name"), "Info", 1)); //$NON-NLS-1$ //$NON-NLS-2$
					//System.out.println("<"+currentDB+">");
					if ((currentDB != null) && (!currentDB.equals(""))) { //$NON-NLS-1$
						if (currentAccess.executeUpdein(("USE ") //$NON-NLS-1$
								.concat(currentDB), true) == true) {
							showImmediatelyRelations = true;
							currentTree.updateTree();
						} else {
							showImmediatelyRelations = false;
							currentTree.updateTree();
							JOptionPane
									.showMessageDialog(
											null,
											Messages
													.getString("MainFrame.dbnotpres"), Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
						}
					} else {
						showImmediatelyRelations = false;
						currentTree.updateTree();
					}
				}
			}
		}
	}

	/** Disconnect from dbms */
	private class DisconnectAction implements ActionListener {

		public void actionPerformed(ActionEvent event) {

			int answer = JOptionPane.showConfirmDialog(null, currentAccess
					.getConnectionData().getUsername()
					+ " " + Messages.getString("MainFrame.logoutquest"), //$NON-NLS-1$
					Messages.getString("MainFrame.Information"), 1); //$NON-NLS-1$
			if (answer == 0) {
				if (!currentAccess.disconnect()) {
					JOptionPane
							.showMessageDialog(
									null,
									Messages
											.getString("MainFrame.error_during_disconnection"), "Error", 0); //$NON-NLS-1$ //$NON-NLS-2$
					return;
				}
				if (currentTree != null) {
					currentTree.doDefaultCloseAction();
				}
				return;
			} else
				return;

		}
	}

}