/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package mysqlconnectus;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.OceanTheme;

import themes.AquaTheme;
import themes.CharcoalTheme;
import themes.ContrastTheme;
import themes.EmeraldTheme;
import themes.RubyTheme;
import tools.FileManager;

/** Used to manage autoconfiguration */
public class SettingsManager {

	/** Pointer to MainFrame */
	final private MainFrame currentParent;

	/** Variable for autoconfig feature */
	protected boolean isJavaStyle = false;

	/** Variable for autoconfig feature */
	protected int javaTheme = 0;

	/** Variable for autoconfig feature */
	protected String style = "";

	/** String rapresenting java theme */
	final protected String javaStyle = "javax.swing.plaf.metal.MetalLookAndFeel";

	/** True if next session will use the java theme */
	protected boolean nextSessionIsJavaStyle = true;

	/** The number associated with a certain java style */
	protected int nextSessionJavaTheme = 1;

	/** Name of the next session theme */
	protected String nextSessionStyle = null;

	/** Name of the next session script */
	protected String nextSessionScript = null;

	/** True if next session use last loaded script */
	protected boolean nextSessionUseLastLoadedScript = false;

	/** True if current session use autoconfiguration */
	protected boolean isAutoConfigEnabled = true;

	/** True if a script is set */
	protected boolean scriptSet = false;

	/** Flag for theme validation */
	protected boolean styleOk = true;

	/** Variable for current theme */
	protected DefaultMetalTheme theme = null;

	/** Configuration table */
	private Properties configProperties = null;

	SettingsManager(final Component ancestor) {
		currentParent = (MainFrame) ancestor;
		configProperties = new Properties();
		FileManager fm = new FileManager();

	}

	/** Method for saving current settings */
	public void saveSettings() {

		if (isAutoConfigEnabled)
			configProperties.setProperty("Auto Configuration",
					"true");
		else
			configProperties.setProperty("Auto Configuration",
					"false");
		if (nextSessionIsJavaStyle)
			configProperties.setProperty(
					"Java Style", "true");
		else
			configProperties.setProperty(
					"Java Style", "false");

		configProperties.setProperty("Java theme : ", (new Integer(
				nextSessionJavaTheme)).toString());

		if (nextSessionStyle != null)
			configProperties.setProperty("Theme of next session ",
					nextSessionStyle);
		else
			configProperties.setProperty("Theme of next session ", javaStyle);

		configProperties.setProperty("script", (new Boolean(
				nextSessionUseLastLoadedScript)).toString());

		if ((nextSessionScript != null) && (nextSessionUseLastLoadedScript))
			configProperties.setProperty("Script path : ", nextSessionScript);

		try {
			configProperties.storeToXML(new FileOutputStream(new File(
					"config.xml")),
					"Auto Configuration Settings of MySQLConnectus 0.4.0");
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, "File not found !", "Warning",
					2);
			return;
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "IO Error !", "Error", 0);
			return;
		}

	}

	/** Check if config file has to be used */
	public boolean useAutoConfig() {
		String temp = null;

		try {
			configProperties.loadFromXML(new FileInputStream(new File(
					"config.xml")));
		}

		catch (InvalidPropertiesFormatException e) {
			JOptionPane
					.showMessageDialog(null,
							"Bad config file\ndefault settings applied !",
							"Warning", 2);
			return false;

		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, "File not found !", "Warning",
					2);
			return false;

		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "IO Error !", "Error", 0);
			return false;
		}

		temp = configProperties.getProperty("Auto Configuration");
		if (temp.equals("true"))
			return true;

		return false;
	}

	/** Set or not autoconfig settings */
	public void autoconfig() {
		if (useAutoConfig()) {
			setAutoConfigSettings();
			//System.out.println(style);
			isAutoConfigEnabled = true;
			//By default nextSession* variables are set like startup values
			nextSessionIsJavaStyle = isJavaStyle;
			if (styleOk) {
				nextSessionJavaTheme = javaTheme;
				nextSessionStyle = style;
			}
			setNewTheme(isJavaStyle, javaTheme, style);

		}
		if (!useAutoConfig()){
			currentParent.mainSplit.setDividerLocation(0);
			isAutoConfigEnabled = false;}
		//long b=System.currentTimeMillis();
		//System.out.println(b-a);

	}

	/** Set autoconfig settings */
	public void setAutoConfigSettings() {

		if (configProperties.getProperty(
				"Java Style").equalsIgnoreCase(
				"false"))
			isJavaStyle = false;

		else if (configProperties.getProperty(
				"Java Style").equalsIgnoreCase(
				"true"))
			isJavaStyle = true;

		else {
			JOptionPane.showMessageDialog(null,
					"Invalid config file\ndefault settings applied !",
					"Warning", 2);
			currentParent.mainSplit.setDividerLocation(0);
			return;
		}

		javaTheme = (new Integer(configProperties.getProperty("Java theme : ")))
				.intValue();

		if (javaTheme > 6) {
			JOptionPane.showMessageDialog(null,
					"Invalid config file\ndefault settings applied !",
					"Warning", 2);
			nextSessionJavaTheme = 1;
			currentParent.mainSplit.setDividerLocation(0);
			return;
		}

		style = configProperties.getProperty("Theme of next session ");

		if ((configProperties.getProperty("script"))
				.equalsIgnoreCase("true"))
			nextSessionUseLastLoadedScript = true;

		else if ((configProperties.getProperty("script"))
				.equalsIgnoreCase("false"))
		{
			nextSessionUseLastLoadedScript = false;
			currentParent.mainSplit.setDividerLocation(0);
		}
		else {
			JOptionPane.showMessageDialog(null,
					"Invalid config file\ndefault settings applied !",
					"Warning", 2);
			return;
		}

		if (configProperties.getProperty("Script path : ") == null
				|| configProperties.getProperty("script")
						.equalsIgnoreCase("false"))
			return;

		nextSessionScript = configProperties.getProperty("Script path : ");
		currentParent.currentAccess.executeScript(nextSessionScript);
				
		if (currentParent.currentTree != null)
			currentParent.currentTree.doDefaultCloseAction();

		currentParent.accessDone = true;
		currentParent.currentTree = new Tree(currentParent);
	
		
		//System.out.println("<"+currentDB+">");
		if ((currentParent.currentDB != null)
				&& (!currentParent.currentDB.equals(""))) {
			if (currentParent.currentAccess.executeUpdein(("USE ")
					.concat(currentParent.currentDB),true) == true) {
				currentParent.showImmediatelyRelations = true;
				currentParent.currentTree.updateTree();
				//System.out.println("Ciao");
			} else {
				currentParent.showImmediatelyRelations = false;
				currentParent.currentTree.updateTree();
				JOptionPane.showMessageDialog(null, "Database not present!",
						"Warning", 2);
			}
		} else {
			currentParent.showImmediatelyRelations = false;
			currentParent.currentTree.updateTree();
		}
	

	}

	/** Set a new theme */
	public void setNewTheme(boolean isJavaStyle, int javaTheme, String style) {
		//System.out.println(javaTheme+" "+isJavaStyle);
		try {
			if (isJavaStyle) {

				if (javaTheme == 1)
					theme = new OceanTheme();
				else if (javaTheme == 2)
					theme = new RubyTheme();
				else if (javaTheme == 3)
					theme = new EmeraldTheme();
				else if (javaTheme == 4)
					theme = new CharcoalTheme();
				else if (javaTheme == 5)
					theme = new ContrastTheme();
				else if (javaTheme == 6)
					theme = new AquaTheme();
				else
					theme = new DefaultMetalTheme();

				if (theme != null) {
					MetalLookAndFeel.setCurrentTheme(theme);

				}
			}
			UIManager.setLookAndFeel(style);
			SwingUtilities.updateComponentTreeUI(currentParent);

		} catch (ClassNotFoundException cnfe) {
			JOptionPane.showMessageDialog(null,
					"This kind of style is not present in your System",
					"Error", 0);
			styleOk = false;
			nextSessionStyle = javaStyle;
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null,
					"Error in loading selected style", "Error", 0);
			styleOk = false;
			nextSessionStyle = javaStyle;
			e.printStackTrace();
		}

	}

}
