/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package mysqlconnectus;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

import tools.Table;
import dbconnectivity.BufferedResultSet;

/**
 * Tree rapresenting database structure
 * 
 * @version 0.2
 */
public class Tree extends JInternalFrame {

	/** Current tree */
	private JTree tree;

	/** Panel where the tree is shown */
	private JPanel treePanel;

	/** The root node */
	private IconNode root;

	/** Pointer to current MainFrame */
	final private MainFrame currentParent;

	/** Temporary string */
	private String temp;

	/** Main constructor */
	public Tree(final Component ancestor) {
		super("Dbms tree");
		setSize(200, 350);
		setLocation(0, 0);
		setFrameIcon(new ImageIcon("Resources/root.gif"));
		currentParent = (MainFrame) ancestor;
		

	}

	/** This method update tree nodes */
	public void updateTree() {

		BufferedResultSet rs2 = null;
		String str = "";
		rs2 = currentParent.currentAccess.executeQuery("SHOW DATABASES");
		int j = 0;
		try {
			while (true) {
				str = str.concat(((String) ((Vector) ((Vector) rs2.doc.get(0))
						.get(j)).get(0)));
				str = str.concat(" ");
				j++;
			}
		} catch (ArrayIndexOutOfBoundsException aioobe) {
		} catch (NullPointerException npe) {
			return;
		}
		treePanel = new JPanel();
		root = new IconNode(currentParent.currentAccess.getConnectionData()
				.getUsername()
				+ "@MySQL", "Resources/root.gif");
		StringTokenizer st = new StringTokenizer(str);
		while (st.hasMoreTokens())
			root.add(new IconNode(st.nextToken(), "Resources/node.gif"));
		tree = new JTree(root);
		tree.setDragEnabled(true);
		tree.setCellRenderer(new TreeIcon());

		MouseListener m2 = new MouseAdapter() {

			public void mouseReleased(MouseEvent e) {
				if (e.getClickCount() == 2) {
					//System.out.println(tree);
					currentParent.currentQuery = null;
					exploreDb(e);
					currentParent.showImmediatelyRelations = true;

				}
			}
		};

		tree.addMouseListener(m2);
		treePanel.add(tree, BorderLayout.CENTER);

		JScrollPane jsp = new JScrollPane(treePanel);
		jsp.setVisible(false);
		getContentPane().add(jsp, BorderLayout.CENTER);
		jsp.setVisible(true);

		this.setClosable(true);
		this.setMaximizable(true);
		this.setIconifiable(true);
		this.setResizable(true);
		currentParent.mainSplit.setLeftComponent(this);
		currentParent.mainSplit.setDividerLocation(200);
		//desktop.add(this, 1);
		this.show();

		//System.out.println("showImmediatelyRelations");

		if (currentParent.showImmediatelyRelations) {
			rs2 = null;
			str = "";

			try {
				rs2 = currentParent.currentAccess.executeQuery("SHOW TABLES");
				j = 0;
				try {
					while (true) {
						str = str.concat(((String) ((Vector) ((Vector) rs2.doc
								.get(0)).get(j)).get(0)));
						str = str.concat(" ");
						j++;
					}
				} catch (ArrayIndexOutOfBoundsException aioobe) {
				}

			}

			catch (NullPointerException npe) {
			}

			IconNode dmtn = root;

			st = new StringTokenizer(str);

			int i = 0;
			if (currentParent.currentDB != null && dmtn != null) {

				while ((!(dmtn.toString()).equals(currentParent.currentDB))) {
					dmtn = (IconNode) dmtn.getNextNode();
					i++;
					if (dmtn == null)
						break;
					//System.out.println(dmtn+" "+currentDB);
				}

			}

			while (st.hasMoreTokens()) {

				if ((temp = st.nextToken()) == null || dmtn == null)
					break;
				dmtn.add(new IconNode(temp, "Resources/JTable.gif"));
			}

			tree.expandRow(i);

			currentParent.showImmediatelyRelations = false;
		}


	}

	/** Used to select db or open relations */
	public void exploreDb(MouseEvent e) {
		//System.out.println(tree+" "+e.getX()+" "+e.getY());
		int selRow = tree.getRowForLocation(e.getX(), e.getY());
		TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
		if (selRow != -1) {

			if (currentParent.accessDone) {
				currentParent.treeFlag = true;
				if (currentParent.currentAccess.executeUpdein(("USE ")
						.concat(selPath.getLastPathComponent().toString()),
						false) == false) {
					BufferedResultSet prs = (currentParent.currentAccess
							.executeQuery("SELECT * FROM "
									+ selPath.getLastPathComponent().toString()));
					currentParent.tab = new Table(prs.doc, prs.columnNames,
							currentParent.frameDesktop, selPath
									.getLastPathComponent().toString(),
							prs.columnType, currentParent);

					currentParent.currentQuery = "SELECT * FROM "
							+ selPath.getLastPathComponent().toString();
					currentParent.treeFlag = false;
					return;
				}

			} else
				JOptionPane
						.showMessageDialog(null, "Not logged in!", "Info", 2);

			System.gc();

			BufferedResultSet rs2 = null;
			String str = "";

			rs2 = currentParent.currentAccess.executeQuery("SHOW TABLES");
			//System.out.println(rs2);
			int j = 0;
			try {
				while (true) {
					str = str.concat(((String) ((Vector) ((Vector) rs2.doc
							.get(0)).get(j)).get(0)));
					str = str.concat(" ");
					j++;
				}
			} catch (ArrayIndexOutOfBoundsException aioobe) {
			}

			IconNode dmtn = root;

			StringTokenizer st = new StringTokenizer(str);
			//System.out.println(str);
			int index = 1;

			while (dmtn != null) {
				if (index != 1) {
					tree.collapseRow(index - 1);
					dmtn.removeAllChildren();
				}
				dmtn = (IconNode) dmtn.getNextNode();
				index++;
			}
			dmtn = root;
			tree.collapseRow(index);
			index = 1;

			while (!(dmtn.toString()).equals(selPath.getLastPathComponent()
					.toString())) {
				dmtn = (IconNode) dmtn.getNextNode();
				index++;
			}

			while (st.hasMoreTokens()) {
				dmtn.add(new IconNode(st.nextToken(), "Resources/JTable.gif"));
			}

			tree.expandRow(index - 1);
			//System.out.println("Row : "+index);
			currentParent.currentDB = selPath.getLastPathComponent().toString();

		}
	}

}