/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package tools;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JOptionPane;

/** Read/write files tools */
public class FileManager {

    /** Flag for file */
    private boolean fnf = false;

    /** Read text */
    public String text;

    /** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages");
	
    /** Default constructor */
    public FileManager() {

    }

    /** Classic read file method */
    public boolean readFile(String filename) {
        String temp = ""; //$NON-NLS-1$
        BufferedReader br = null;
        String tmp = null;
        fnf = false;

        try {
            br = new BufferedReader(new FileReader(filename));
            temp = ""; //$NON-NLS-1$
            while ((tmp = br.readLine()) != null) {
                temp = temp.concat(tmp);
                temp = temp.concat("\n "); //$NON-NLS-1$

            }
            text = temp;
        } catch (FileNotFoundException fnfe) {
            JOptionPane.showMessageDialog(null, "File '" + filename //$NON-NLS-1$
                    + Messages.getString("MainFrame.notpresent"), Messages.getString("MainFrame.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
            fnf = true;
            return fnf;
        } catch (IOException ioe) {
            ioe.printStackTrace();
            JOptionPane.showMessageDialog(null, Messages.getString("FileManager.6"), Messages.getString("MainFrame.Error"), //$NON-NLS-1$ //$NON-NLS-2$
                    0);
            fnf = true;
            return fnf;
        } finally {
            if (br != null) {
                try {
                    br.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }

        return fnf;
    }

    /** Method for saving a string file */
    public boolean saveFile(String text, String name) {

    	File fname = new File(name);
    	if (fname.exists())
    	{
    		if ( JOptionPane.showConfirmDialog(null,Messages.getString("FileManager.overwrite")) != 0 ) //$NON-NLS-1$
    		{
    			return false;
    		}
    	}
        PrintWriter out = null;
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(name)));
            out.println(text);

            if (out != null) {
                out.flush();
                out.close();
            }

        } catch (IOException ioe) {
            return false;
        }

        return true;
    }

    /** Method for saving a file that is an exportedToTxt table */
    public boolean saveFile(Vector cnames, Vector savedDoc, String name) {
    	
    	File fname = new File(name);
    	//System.out.println(fname);
    	if (fname.exists())
    	{
    		if ( JOptionPane.showConfirmDialog(null,Messages.getString("FileManager.overwrite")) != 0 ) //$NON-NLS-1$
    		{
    			return false;
    		}
    	}
        
        PrintWriter out = null;
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(name)));
            String tmp = ""; //$NON-NLS-1$
            int columnCount = cnames.size();
            for (int i = 0; i < columnCount; i++) {
                tmp = tmp.concat((String) cnames.get(i) + "\t"); //$NON-NLS-1$
                //   System.out.println((String)cnames.get(i));
            }
            out.println(tmp);
            tmp = ""; //$NON-NLS-1$
            for (int i = 0; i < savedDoc.size(); i++) {
                for (int j = 0; j < ((Vector) savedDoc.get(i)).size(); j++) {
                    for (int k = 0; k < columnCount; k++) {
                        tmp = tmp.concat(((String) ((Vector) ((Vector) savedDoc
                                .get(i)).get(j)).get(k))
                                + "\t"); //$NON-NLS-1$
                        // System.out.println(((String)((Vector)((Vector)savedDoc.get(i)).get(j)).get(k))+""+j+""+k+""+i+"\n");
                    }
                    out.println(tmp);
                    tmp = ""; //$NON-NLS-1$
                }

            }

            if (out != null) {
                out.flush();
                out.close();
            }

        } catch (IOException ioe) {
            return false;
        }

        return true;
    }

    /**Method for copying a text file */
    public boolean copy(String source,String destination,boolean noFirstLine)
    {
    	PrintWriter out=null;
    	BufferedReader br = null;
    	
    	try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(destination)));
            br = new BufferedReader(new FileReader(source));
            String tmp = ""; //$NON-NLS-1$
            int i=0;
            while ((tmp = br.readLine()) != null) {
                if (i!=0 || !noFirstLine)
                {
                	out.print(tmp);
                	out.print("\n"); //$NON-NLS-1$
                }
                i++;
            }
            if (out != null) {
                out.flush();
                out.close();
            }

    	}
    	 catch (IOException ioe) {
            return false;
        }
    	 
    	return true;
    }
    /** Method for saving metadata or in general vector of strings*/
    public boolean saveFile(Vector metadata, String name) {
        
    	File fname = new File(name);
    	if (fname.exists())
    	{
    		if ( JOptionPane.showConfirmDialog(null,Messages.getString("FileManager.overwrite")) != 0 ) //$NON-NLS-1$
    		{
    			return false;
    		}
    	}
        
    	PrintWriter out = null;
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(name)));
            if (metadata==null)
                return false;
            int size = metadata.size();
            for (int i = 0; i < size; i++) {
                out.println((String) metadata.get(i));
            }

            if (out != null) {
                out.flush();
                out.close();
            }

        } catch (IOException ioe) {
            return false;
        }

        return true;

    }
}