/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package dbconnectivity;

import java.awt.Component;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;

import mysqlconnectus.MainFrame;
import mysqlconnectus.Tree;
import tools.FileManager;

/**
 * Core class of the program. Axxess is the class used for interacting with the
 * MySQL DBMS. SINGLETON pattern used.
 * 
 * @author Sergio Decherchi
 * @version 0.2
 * @see BufferedResultSet 
 * @see DatabaseSource 
 * @see DatabaseConnector
 *  
 */

public class Axxess implements DatabaseConnector {

	/** Connection instance */
	private Connection currentConnection = null;

	/** Statement that has to be executed */
	private Statement stmt = null;

	/** Result of the query */
	private ResultSet rs = null;

	/** This inidcates the exitus to an access to the DB */
	protected String exitusStatement = null;

	/** Host name or IP address (default is localhost 127.0.0.1) */
	private String hostname = "127.0.0.1"; //$NON-NLS-1$

	/** Pointer to current MainFrame */
	private MainFrame currentParent;

	/** Unique instance of Axxess (Singleton pattern used) */
	public static Axxess uniqueInstance = null;

	/** Connection data */
	private ConnectionData cdata = new ConnectionData();

	/** Resource bundle for localization */
	private ResourceBundle Messages = ResourceBundle.getBundle("Lang/messages");

	/** Good connection */
	private String goodConnection = Messages.getString("Axxess.GoodConnection"); //$NON-NLS-1$

	/** Main constructor */
	private Axxess() {
		exitusStatement = goodConnection;

	}

	/** Static method used for getting the unique instance of the class */
	static Axxess getInstance() {

		if (uniqueInstance == null) {
			uniqueInstance = new Axxess();
			return uniqueInstance;

		} else
			return uniqueInstance;

	}

	/** Set current parent component */
	public void setAncestor(Component ancestor) {
		currentParent = (MainFrame) ancestor;
	}

	/**
	 * Method to access DBMS
	 * 
	 * @param us
	 *            username
	 * @param pass
	 *            password
	 * @param hostname
	 *            hostname or IP address
	 * @return Access statement :
	 *         <P>
	 *         "Successfully connected!" for a correct connection
	 *         <P>
	 *         "Connection failed!" for incorrect connection if is used english
	 *         localization
	 */

	public String access(String us, String pass, String hostname) {

		exitusStatement = goodConnection;
		String temp = ""; //$NON-NLS-1$

		try {

			Class.forName("com.mysql.jdbc.Driver").newInstance(); //$NON-NLS-1$

		} catch (Exception ex) {
			JOptionPane.showMessageDialog(null, Messages
					.getString("Axxess.ClasspathError"), //$NON-NLS-1$
					Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$
		}
		if (hostname != null) {
			this.hostname = hostname;
		}

		temp = "jdbc:mysql://" + this.hostname + "/"; //$NON-NLS-1$ //$NON-NLS-2$

		temp = temp.concat("?user="); //$NON-NLS-1$
		temp = temp.concat(us);
		temp = temp.concat("&password="); //$NON-NLS-1$
		temp = temp.concat(pass);

		try {

			currentConnection = DriverManager.getConnection(temp);

		}

		catch (SQLException ex)

		{
			String temp2 = ""; //$NON-NLS-1$
			String tmp = ex.getMessage();
			StringTokenizer st = new StringTokenizer(tmp);
			int i = 0;
			while (st.hasMoreTokens()) {
				temp2 = temp2.concat(st.nextToken() + " "); //$NON-NLS-1$
				if (temp2.length() > 7) {
					tmp = temp2.substring(0, 7);

				}
				if (i > 7) {
					temp2 = temp2.concat("\n"); //$NON-NLS-1$
					i = 0;
				}
				i++;
			}
			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("Axxess.BadConnection"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
			return exitusStatement = Messages
					.getString("Axxess.ConnectionFailed"); //$NON-NLS-1$
		}

		try {

			stmt = currentConnection.createStatement();

		} catch (SQLException ex2) {
			String temp2 = ""; //$NON-NLS-1$
			String tmp = ex2.getMessage();
			StringTokenizer st = new StringTokenizer(tmp);
			int i = 0;
			while (st.hasMoreTokens()) {
				temp2 = temp2.concat(st.nextToken() + " "); //$NON-NLS-1$
				if (i > 7) {
					temp2 = temp2.concat("\n"); //$NON-NLS-1$
					i = 0;
				}
				i++;
			}

			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("Axxess.SQLException") + temp //$NON-NLS-1$
									+ Messages.getString("Axxess.SQLState") + ex2.getSQLState() + Messages.getString("Axxess.VendorError") //$NON-NLS-1$ //$NON-NLS-2$
									+ ex2.getErrorCode(), Messages
									.getString("Axxess.ServerMessage"), 2); //$NON-NLS-1$
			return exitusStatement = Messages
					.getString("Axxess.ConnectionFailed"); //$NON-NLS-1$
		}
		// System.out.println("update");

		if (hostname != null)
			cdata.setHost(hostname);
		else
			cdata.setHost("localhost"); //$NON-NLS-1$
		cdata.setUsername(us);
		cdata.setPassword(pass);
		return exitusStatement;
	}

	/** Disconnect from dbms */
	public boolean disconnect() {
		try {
			stmt.close();
			currentConnection.close();
		} catch (SQLException sqle) {
			return false;
		}
		cdata.setHost(null);
		cdata.setUsername(null);
		cdata.setPassword(null);
		return true;
	}

	/** Method for executing a script */
	public boolean executeScript(String filename) {

		cdata.setUsername(null);
		cdata.setPassword(null);
		String dbname = null;
		String temp = null;
		int i = 0;
		FileManager fm = new FileManager();
		//System.out.println(filename);
		if (fm.readFile(filename)) {
			JOptionPane.showMessageDialog(null, Messages
					.getString("Axxess.ScriptNotLoaded"), //$NON-NLS-1$
					Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$
			return false;
		}
		//System.out.println(fm.text);
		StringTokenizer st = new StringTokenizer(fm.text);

		while (st.hasMoreTokens()) {
			temp = st.nextToken();
			//System.out.println(temp);
			if (i == 0) {
				//System.out.println((temp.substring(0,6)));
				try {
					if ((temp.substring(0, 6)).equals("HOST->")) { //$NON-NLS-1$
						//System.out.println(temp.substring(6,temp.length()));
						cdata.setHost(temp.substring(6, temp.length()));
					} else {
						JOptionPane
								.showMessageDialog(
										null,
										Messages.getString("Axxess.BFSF"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
						return false;
					}
				} catch (StringIndexOutOfBoundsException e) {
					JOptionPane
							.showMessageDialog(
									null,
									Messages.getString("Axxess.BFSF"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
					return false;
				}

			}

			if (i == 1) {
				//System.out.println((temp.substring(0,6)));
				if ((temp.substring(0, 6)).equals("USER->")) { //$NON-NLS-1$
					//System.out.println(temp.substring(6,temp.length()));
					cdata.setUsername(temp.substring(6, temp.length()));
				} else {
					JOptionPane
							.showMessageDialog(
									null,
									Messages.getString("Axxess.BFSF"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
					return false;
				}

			}
			if (i == 2) {
				if (temp.startsWith("PASSWORD->")) { //$NON-NLS-1$
					if ((temp.substring(10, temp.length())).equals("null")) //$NON-NLS-1$
						cdata.setPassword(""); //$NON-NLS-1$
					else
						cdata.setPassword(temp.substring(10, temp.length()));
					//System.out.println(password);
				}

				else {
					JOptionPane
							.showMessageDialog(
									null,
									Messages.getString("Axxess.BFSF"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
					return false;
				}

			}
			if (i == 3) {
				if (temp.startsWith("DBNAME->")) { //$NON-NLS-1$
					dbname = temp.substring(8, temp.length());
					currentParent.setCurrentDB(dbname);
				}

				else {
					JOptionPane
							.showMessageDialog(
									null,
									Messages.getString("Axxess.BFSF"), Messages.getString("Axxess.Error"), 0); //$NON-NLS-1$ //$NON-NLS-2$
					return false;
				}
				//System.out.println(dbname);
				//System.out.println(temp.substring(8,temp.length()));
			}

			i++;

		}
		if ((cdata.getUsername() != null) && (cdata.getPassword() != null)) {
			String returnStmt = this.access(cdata.getUsername(), cdata
					.getPassword(), null);
			if (returnStmt
					.equals(Messages.getString("Axxess.ConnectionFailed"))) //$NON-NLS-1$
			{
				return false;
			}

		} else {
			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("Axxess.IF"), Messages.getString("Axxess.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}

		if ((dbname != null) && (!dbname.equalsIgnoreCase("")) //$NON-NLS-1$
				&& (!dbname.equalsIgnoreCase(" "))) { //$NON-NLS-1$

			if (currentParent.currentTree != null) {
				if ((this.executeUpdein("USE " + dbname, true)) == false) { //$NON-NLS-1$
					currentParent.showImmediatelyRelations = false;
					JOptionPane
							.showMessageDialog(
									null,
									"Database '" + dbname //$NON-NLS-1$
											+ Messages
													.getString("Axxess.notpresent"), Messages.getString("Axxess.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
				} else
					currentParent.showImmediatelyRelations = true;

				currentParent.currentTree = new Tree(currentParent);
				currentParent.currentTree.updateTree();
			}

		}

		else {
			JOptionPane.showMessageDialog(null, Messages
					.getString("Axxess.DBnotspec"), //$NON-NLS-1$
					Messages.getString("Axxess.W"), 2); //$NON-NLS-1$
			currentParent.showImmediatelyRelations = false;
			currentParent.currentTree.updateTree();
			currentParent.accessDone = true;
			return false;
		}

		currentParent.accessDone = true;
		return true;

	}

	/** Method for executing a modification */
	public boolean executeUpdein(String query, boolean warning) {

		try {

			try {
				stmt.executeUpdate(query);
			} catch (NullPointerException npe) {
				JOptionPane
						.showMessageDialog(
								null,
								Messages.getString("Axxess.DBnotsel"), Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}

		catch (SQLException ex) {
			String temp = ""; //$NON-NLS-1$
			String tmp = ex.getMessage();
			StringTokenizer st = new StringTokenizer(tmp);
			int i = 0;
			while (st.hasMoreTokens()) {
				temp = temp.concat(st.nextToken() + " "); //$NON-NLS-1$
				if (i > 7) {
					temp = temp.concat("\n"); //$NON-NLS-1$
					i = 0;
				}
				i++;
			}
			//if (!currentParent.treeFlag)
			if (warning) {
				//System.out.println("CIAO");
				JOptionPane
						.showMessageDialog(
								null,
								Messages.getString("Axxess.SQLException") + temp //$NON-NLS-1$
										+ Messages.getString("Axxess.SQLState") + ex.getSQLState() //$NON-NLS-1$
										+ Messages
												.getString("Axxess.VendorError") + ex.getErrorCode(), //$NON-NLS-1$
								Messages.getString("Axxess.ServerMessage"), 2); //$NON-NLS-1$
				return false;
			} else
				return false;
		} catch (Exception e) {
			if (warning) {
				JOptionPane.showMessageDialog(null, Messages
						.getString("Axxess.Exception") + e.toString(), //$NON-NLS-1$
						Messages.getString("Axxess.ServerMessage"), 2); //$NON-NLS-1$
				return false;
			} else
				return true;
		}

		return true;

	}
	/**
	 * Return connection data
	 * 
	 * @return connection data
	 */
	public ConnectionData getConnectionData() {
		return cdata;
	}

	/** Method for executing a query */
	public BufferedResultSet executeQuery(String query) {
		try {

			rs = stmt.executeQuery(query);
		} catch (SQLException ex) {
			String temp = ""; //$NON-NLS-1$
			String tmp = ex.getMessage();
			StringTokenizer st = new StringTokenizer(tmp);
			int i = 0;
			while (st.hasMoreTokens()) {
				temp = temp.concat(st.nextToken() + " "); //$NON-NLS-1$
				if (i > 7) {
					temp = temp.concat("\n"); //$NON-NLS-1$
					i = 0;
				}
				i++;
			}

			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("Axxess.SQLException") + temp //$NON-NLS-1$
									+ Messages.getString("Axxess.SQLState") + ex.getSQLState() + Messages.getString("Axxess.VendorError") //$NON-NLS-1$ //$NON-NLS-2$
									+ ex.getErrorCode(), Messages
									.getString("Axxess.ServerMessage"), 2); //$NON-NLS-1$
			//System.out.println("doh!");
			return null;
		} catch (NullPointerException npe) {
			JOptionPane
					.showMessageDialog(
							null,
							Messages.getString("Axxess.DBnotsel"), Messages.getString("Axxess.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
			return null;

		}

		BufferedResultSet pr = new BufferedResultSet(rs);
		//pr.setData();
		return pr;

	}

}