/*
 * Created on 9-ott-2004 at 11.40.35 by SedekFx
 * 
 * This file is part of MySQLConnectus by Sergio Decherchi
 * 
 * Copyright (C) 2004 Sergio Decherchi
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details (gpl.txt). You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package mysqlconnectus;

import java.awt.Component;
import java.io.File;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import tools.FileManager;
import dbconnectivity.BufferedResultSet;

/**Class that manages table import methods */
public class ImportTable {

    /** Pointer to current parent frame */
    final private MainFrame currentParent;

    /** " char */
    static private char dquote = '"';

    /** file name and meta file name */
    public String file, metafile;

    /** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages");
	
    /** Default constructor */
    public ImportTable() {
        currentParent = null;
    }

    /** Constructor with param parent component */
    public ImportTable(Component c) {
        currentParent = (MainFrame) c;
    }

    

    /** Import data from txt file to selected table */
    public void importFromTxt() {
        String file = null;
        JFileChooser chooser = new JFileChooser();
        chooser.setCurrentDirectory(new File(".")); //$NON-NLS-1$
        int r = chooser.showOpenDialog(null);
        if (r == JFileChooser.APPROVE_OPTION) {
            file = chooser.getSelectedFile().getAbsolutePath();
            //file = chooser.getSelectedFile().getName();
            if (file == null) {
                JOptionPane.showMessageDialog(null, Messages.getString("ImportTable.1"), //$NON-NLS-1$
                        Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
                return;
            }

            BufferedResultSet brs = currentParent.currentAccess
                    .executeQuery("SHOW TABLES"); //$NON-NLS-1$
            Object[] possibleValues = new Object[((Vector) ((Vector) brs.doc
                    .get(0))).size()];
            for (int i = 0; i < ((Vector) ((Vector) brs.doc.get(0))).size(); i++)
                possibleValues[i] = ((String) ((Vector) ((Vector) brs.doc
                        .get(0)).get(i)).get(0));
            Object selectedValue = JOptionPane.showInputDialog(null,
                    Messages.getString("ImportTable.4"), Messages.getString("ImportTable.5"), //$NON-NLS-1$ //$NON-NLS-2$
                    JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                    possibleValues[0]);
            if (selectedValue == null) { return; }
            file = file.substring(2, file.length());
            file = file.replace('\\', '/');
            FileManager fm = new FileManager();
            fm.copy(file,file+".txt",true); //$NON-NLS-1$
            //System.out.println(file);
            boolean b = currentParent.currentAccess
                    .executeUpdein("LOAD DATA LOCAL INFILE " + dquote + file+".txt" //$NON-NLS-1$ //$NON-NLS-2$
                            + dquote + " INTO TABLE " //$NON-NLS-1$
                            + ((String) selectedValue) + " ;",true); //$NON-NLS-1$
            if (!b) {
                JOptionPane.showMessageDialog(null, Messages.getString("HtmlHelpOnLine.7"), //$NON-NLS-1$
                        Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
            }
            return;
        }
        System.gc();
    }

    /** Import data from currently shown table to selected table */
    public void importFromTxtDirect() {
        BufferedResultSet brs = currentParent.currentAccess
                .executeQuery("SHOW TABLES"); //$NON-NLS-1$
        Object[] possibleValues = new Object[((Vector) ((Vector) brs.doc.get(0)))
                .size()];
        for (int i = 0; i < ((Vector) ((Vector) brs.doc.get(0))).size(); i++)
            possibleValues[i] = ((String) ((Vector) ((Vector) brs.doc.get(0))
                    .get(i)).get(0));
        Object selectedValue = JOptionPane.showInputDialog(null,
                Messages.getString("ImportTable.4"), Messages.getString("ImportTable.5"), //$NON-NLS-1$ //$NON-NLS-2$
                JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                possibleValues[0]);
        if (selectedValue == null) { return; }
        file = file.replace('\\', '/');
        FileManager fm = new FileManager();
        fm.copy(file,file+".txt",true); //$NON-NLS-1$
        //System.out.println(file);
        boolean b = currentParent.currentAccess
                .executeUpdein("LOAD DATA LOCAL INFILE " + dquote + file+".txt" //$NON-NLS-1$ //$NON-NLS-2$
                        + dquote + " INTO TABLE " + ((String) selectedValue) //$NON-NLS-1$
                        + " ;",true); //$NON-NLS-1$
        if (!b) {
            JOptionPane
                    .showMessageDialog(null, Messages.getString("HtmlHelpOnLine.7"), Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$ //$NON-NLS-2$
        }
        System.gc();
        return;
    }

    /** Import&Create table from filename.txt and filename@metadata.txt */
    public void importCreateFromTxt() {
        String tableName, creationStatement, metafileText, fileText, tmp = null;
        creationStatement = ""; //$NON-NLS-1$
        JFileChooser chooser = new JFileChooser();
        chooser.setCurrentDirectory(new File(".")); //$NON-NLS-1$
        int r = chooser.showOpenDialog(null);
        if (r == JFileChooser.APPROVE_OPTION) {
            file = chooser.getSelectedFile().getAbsolutePath();
            //file = chooser.getSelectedFile().getName();
            if (file == null) {
                JOptionPane.showMessageDialog(null, Messages.getString("ImportTable.1"), //$NON-NLS-1$
                        Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
                return;
            }
            metafile = (file.substring(0, (file.length() - 4)))
                    .concat("@metadata.txt"); //$NON-NLS-1$
            FileManager fm = new FileManager();
            if (fm.readFile(metafile)) { 
                JOptionPane.showMessageDialog(null, Messages.getString("ImportTable.28"), //$NON-NLS-1$
                        Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$
                
                return; }
            metafileText = fm.text;
            if (fm.readFile(file)) { return; }
            fileText = fm.text;
            StringTokenizer st = new StringTokenizer(fileText);
            StringTokenizer st2 = new StringTokenizer(metafileText);
            while (st2.hasMoreTokens()) {
                tmp = ""; //$NON-NLS-1$
                tmp = st2.nextToken();
                if (tmp.equals("VARCHAR") || tmp.equals("CHAR")) { //$NON-NLS-1$ //$NON-NLS-2$
                    tmp = tmp.concat("(100)"); //$NON-NLS-1$
                }
                creationStatement = creationStatement.concat(st.nextToken()
                        + " " + tmp + ", "); //$NON-NLS-1$ //$NON-NLS-2$
            }
            creationStatement = creationStatement.substring(0,
                    creationStatement.length() - 1);
            tableName = (JOptionPane
                    .showInputDialog(
                            null,
                            Messages.getString("ImportTable.36"), //$NON-NLS-1$
                            Messages.getString("MainFrame.Information"), 1)); //$NON-NLS-1$
            if (tableName == null || tableName.equals("")) return; //$NON-NLS-1$

            creationStatement = "CREATE TABLE " + tableName + " ( " //$NON-NLS-1$ //$NON-NLS-2$
                    + creationStatement + " ); "; //$NON-NLS-1$
            currentParent.currentAccess.executeUpdein(creationStatement,true);
            file = file.substring(2, file.length());
            file = file.replace('\\', '/');
            fm.copy(file,file+".txt",true); //$NON-NLS-1$
            //System.out.println(file);
            currentParent.currentAccess.executeUpdein("LOAD DATA LOCAL INFILE " //$NON-NLS-1$
                    + dquote + file+".txt" + dquote + " INTO TABLE " + tableName //$NON-NLS-1$ //$NON-NLS-2$
                    + " ;",true); //$NON-NLS-1$

        }
    }

    /** Import&Create table from currently shown table */
    public void importCreateFromTxtDirect() {
        String tableName, creationStatement, metafileText, fileText, tmp = null;
        creationStatement = ""; //$NON-NLS-1$
        metafile = (file.substring(0, (file.length() - 4)))
                .concat("@metadata.txt"); //$NON-NLS-1$
        FileManager fm = new FileManager();
        if (fm.readFile(metafile)) { return; }
        metafileText = fm.text;
        if (fm.readFile(file)) { return; }
        fileText = fm.text;
        StringTokenizer st = new StringTokenizer(fileText);
        StringTokenizer st2 = new StringTokenizer(metafileText);
        while (st2.hasMoreTokens()) {
            tmp = ""; //$NON-NLS-1$
            tmp = st2.nextToken();
            if (tmp.equals("VARCHAR") || tmp.equals("CHAR")) { //$NON-NLS-1$ //$NON-NLS-2$
                tmp = tmp.concat("(100)"); //$NON-NLS-1$
            }
            creationStatement = creationStatement.concat(st.nextToken() + " " //$NON-NLS-1$
                    + tmp + ", "); //$NON-NLS-1$
        }
        creationStatement = creationStatement.substring(0, creationStatement
                .length() - 1);
        tableName = (JOptionPane.showInputDialog(null,
                Messages.getString("ImportTable.36"), //$NON-NLS-1$
                Messages.getString("MainFrame.Information"), 1)); //$NON-NLS-1$
        if (tableName == null || tableName.equals("")) return; //$NON-NLS-1$
        creationStatement = "CREATE TABLE " + tableName + " ( " //$NON-NLS-1$ //$NON-NLS-2$
                + creationStatement + " ); "; //$NON-NLS-1$
        currentParent.currentAccess.executeUpdein(creationStatement,true);

        file = file.replace('\\', '/');
        fm.copy(file,file+".txt",true); //$NON-NLS-1$
        currentParent.currentAccess.executeUpdein("LOAD DATA LOCAL INFILE " //$NON-NLS-1$
                + dquote + file+".txt" + dquote + " INTO TABLE " + tableName + " ;",true); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    }
}