/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package mysqlconnectus;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.ResourceBundle;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

/**
 * This class rapresents the window in which you can insert your custom
 * AuotoConfig settings
 */

public class OptionPanel extends JInternalFrame {

    /** Pointer to current instance */
    final protected OptionPanel thisOptionPanel = this;

    /** Different options buttons */
    private JRadioButton enableAuto, disableAuto, enableScript, disableScript;

    /** Set script by choosing the file */
    private JButton setManually;

    /** Flags */
    private boolean tempIsAutoConfigEnabled,
            tempNextSessionUseLastLoadedScript, tempScriptSet;

    /** Temporary script for next session ("temporary" means before pressing ok) */
    private String tempNextSessionScript;

    /** Pointer to current MainFrame */
    final private MainFrame currentParent;
    
    /** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages"); //$NON-NLS-1$

    /** Main constructor */
    OptionPanel(final Component ancestor, JDesktopPane desktop) {
        currentParent = (MainFrame) ancestor;
        setTitle(Messages.getString("OptionPanel.0")); //$NON-NLS-1$
        setLocation(0, 0);
        Container contentPane = getContentPane();
        desktop.add(this, 1);
        JPanel pan = new JPanel();
        pan.setBorder(new TitledBorder(null, Messages.getString("OptionPanel.0"), //$NON-NLS-1$
                TitledBorder.LEFT, TitledBorder.TOP));
        enableAuto = new JRadioButton(Messages.getString("OptionPanel.2")); //$NON-NLS-1$
        disableAuto = new JRadioButton(Messages.getString("OptionPanel.3")); //$NON-NLS-1$
        ButtonGroup jbg = new ButtonGroup();
        jbg.add(enableAuto);
        jbg.add(disableAuto);
        setManually = new JButton(Messages.getString("OptionPanel.4")); //$NON-NLS-1$

        disableAuto.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (((JRadioButton) e.getSource()).isSelected()) {
                    tempIsAutoConfigEnabled = false;
                    //  (thisOptionPanel.enableAuto).setSelected(false);
                    (thisOptionPanel.enableScript).setEnabled(false);
                    (thisOptionPanel.disableScript).setEnabled(false);
                    (thisOptionPanel.setManually).setEnabled(false);

                }
                //invalidate();
                //validate();
                //repaint();
            }
        });

        enableAuto.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (((JRadioButton) e.getSource()).isSelected()) {
                    tempIsAutoConfigEnabled = true;
                    //   (thisOptionPanel.disableAuto).setSelected(false);
                    (thisOptionPanel.enableScript).setEnabled(true);
                    (thisOptionPanel.disableScript).setEnabled(true);
                    (thisOptionPanel.setManually).setEnabled(true);
                    if ((thisOptionPanel.disableScript).isSelected())
                            (thisOptionPanel.setManually).setEnabled(false);
                }
                //invalidate();
                //validate();
                //repaint();
            }
        });
        pan.add(enableAuto);
        pan.add(disableAuto);

        JPanel pan2 = new JPanel();
        JPanel pan3 = new JPanel();
        pan2.setBorder(new TitledBorder(null, Messages.getString("OptionPanel.5"), //$NON-NLS-1$
                TitledBorder.LEFT, TitledBorder.TOP));
        enableScript = new JRadioButton(Messages.getString("OptionPanel.6")); //$NON-NLS-1$
        enableScript.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (((JRadioButton) e.getSource()).isSelected()) {
                    tempNextSessionUseLastLoadedScript = true;
                    //     (thisOptionPanel.disableScript).setSelected(false);
                    (thisOptionPanel.setManually).setEnabled(true);
                }
                //invalidate();
                //validate();
                //repaint();
            }
        });
        disableScript = new JRadioButton(Messages.getString("OptionPanel.7")); //$NON-NLS-1$
        disableScript.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (((JRadioButton) e.getSource()).isSelected()) {
                    tempNextSessionUseLastLoadedScript = false;
                    tempNextSessionScript = null;
                    //  (thisOptionPanel.enableScript).setSelected(false);
                    (thisOptionPanel.setManually).setEnabled(false);
                }
                //invalidate();
                //validate();
                //repaint();
            }
        });
        ButtonGroup jbg2 = new ButtonGroup();
        jbg2.add(enableScript);
        jbg2.add(disableScript);

        setManually.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent evt) {
                tempNextSessionUseLastLoadedScript = true;
                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory(currentParent.lastAccessedDirectory); //$NON-NLS-1$
                int r = chooser.showOpenDialog(null);
                if (r == JFileChooser.APPROVE_OPTION) {
                	currentParent.lastAccessedDirectory=chooser.getSelectedFile();
                    tempNextSessionScript = chooser.getSelectedFile()
                            .getAbsolutePath();

                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    Messages.getString("OptionPanel.9"), //$NON-NLS-1$
                                    Messages.getString("MainFrame.Information"), 1); //$NON-NLS-1$
                    tempScriptSet = true;
                }

            }

        });
        JButton ok = new JButton(Messages.getString("PasswordChooser.1")); //$NON-NLS-1$
        JButton cancel = new JButton(Messages.getString("PasswordChooser.3")); //$NON-NLS-1$
        ok.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                currentParent.settings.isAutoConfigEnabled = tempIsAutoConfigEnabled;
                currentParent.settings.nextSessionUseLastLoadedScript = tempNextSessionUseLastLoadedScript;
                currentParent.settings.nextSessionScript = tempNextSessionScript;
                currentParent.settings.scriptSet = tempScriptSet;
                thisOptionPanel.doDefaultCloseAction();
            }
        });
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                thisOptionPanel.doDefaultCloseAction();
            }
        });

        enableScript.setEnabled(false);
        disableScript.setEnabled(false);
        setManually.setEnabled(false);
        pan2.add(enableScript);
        pan2.add(disableScript);
        pan2.add(setManually);
        pan3.setLocation(50, 50);
        pan3.add(ok);
        pan3.add(cancel);
        pan.add(pan2);
        pan.add(pan3);

        //Initialization with previous values.
        if (currentParent.settings.isAutoConfigEnabled) {
            enableAuto.setSelected(true);
            disableAuto.setSelected(false);
            (thisOptionPanel.enableScript).setEnabled(true);
            (thisOptionPanel.disableScript).setEnabled(true);
            (thisOptionPanel.setManually).setEnabled(true);

            if (currentParent.settings.nextSessionUseLastLoadedScript) {
                enableScript.setSelected(true);
                disableScript.setSelected(false);
            } else {
                enableScript.setSelected(false);
                disableScript.setSelected(true);
            }

        } else {
            enableAuto.setSelected(false);
            disableAuto.setSelected(true);
        }
        contentPane.add(pan);
        this.show();
        this.setSize(450, 220);
    }

}