
/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package mysqlconnectus;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.plaf.metal.MetalLookAndFeel;

import tools.User;

/** Class that manages the choosing of the password and username */

public class PasswordChooser extends JPanel {

    /** 1st Text field */
    private JTextField username;

    /** Password field */
    private JPasswordField pass;

    /** IP field */
    private JTextField IP;

    /** Ok button */
    public JButton okButton;

    /** Flag */
    boolean ok;

    /** Current dialog */
    private JDialog dialog;

    /** Resource bundle for localization */
	private ResourceBundle Messages;
	
    /** Pointer to current MainFrame */
    final private MainFrame currentParent;

    PasswordChooser(final Component ancestor) {
    	
    	Messages=ResourceBundle.getBundle("Lang/messages");
        currentParent = (MainFrame) ancestor;
        setLayout(new BorderLayout());
        JPanel panel2 = new JPanel();
        panel2.setLayout(new GridLayout(4, 4));
        panel2.add(new JLabel(Messages.getString("PasswordChooser.0"))); //$NON-NLS-1$
        panel2.add(username = new JTextField("")); //$NON-NLS-1$
        panel2.add(new JLabel(Messages.getString("PasswordChooser.2"))); //$NON-NLS-1$
        panel2.add(pass = new JPasswordField("")); //$NON-NLS-1$
        panel2.add(new JLabel(Messages.getString("PasswordChooser.4"))); //$NON-NLS-1$
        panel2.add(IP = new JTextField("")); //$NON-NLS-1$
        add(panel2, BorderLayout.CENTER);
        okButton = new JButton(Messages.getString("PasswordChooser.1")); //$NON-NLS-1$
        okButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                ok = true;
                dialog.setVisible(false);

            }
        });
        JButton cancelButton = new JButton(Messages.getString("PasswordChooser.3")); //$NON-NLS-1$
        cancelButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                dialog.setVisible(false);
            }
        });
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        add(buttonPanel, BorderLayout.SOUTH);
    }

    public void setUser(User u) {
        username.setText(u.getName());
    }

    public User getUser() {
        return new User(username.getText(), new String(pass.getPassword()));
    }

    public String getIP() {
        return IP.getText();
    }

    public boolean showDialog(Component parent, String title) {
        ok = false;
        Frame owner = null;
        if (parent instanceof Frame)
            owner = (Frame) parent;
        else
            owner = (Frame) SwingUtilities.getAncestorOfClass(Frame.class,
                    parent);
        if (dialog == null || dialog.getOwner() != owner) {
            dialog = new JDialog(owner, true);
            Container p = dialog.getContentPane();
            p.setLayout(new BoxLayout((dialog.getContentPane()),
                    BoxLayout.Y_AXIS));
            p.add(this);
            dialog.getRootPane().setDefaultButton(okButton);
            dialog.pack();

        }
        dialog.setTitle(title);
        dialog.setSize(250, 170);
        dialog.setLocation(150, 150);

        if (currentParent.settings.theme != null) {
            MetalLookAndFeel.setCurrentTheme(currentParent.settings.theme);

        }
        SwingUtilities.updateComponentTreeUI(this);
        dialog.setVisible(true);
        return ok;
    }

}