/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package mysqlconnectus;

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.*;

import javax.swing.*;

import javax.swing.text.*;

import com.Ostermiller.Syntax.Lexer.*;

import tools.*;

import dbconnectivity.*;

/**
 * Frame in which you can insert your query or an insert/upadate/delete
 * statement
 */
class TextEditFrame extends JInternalFrame {

		
	/** Type of text editframe (modification or query) */
	private boolean typ;

	/** Text area */
	private JTextPane textArea;
	
	/**
     * the styled document that is the model for
     * the textPane
     */
    protected HighLightedDocument document;
    
    /**
     * A hash table containing the text styles.
     * Simple attribute sets are hashed by name (String)
     */
    
    private Hashtable styles = new Hashtable();
    
    /**
     * A reader wrapped around the document
     * so that the document can be fed into
     * the lexer.
     */
    protected DocumentReader documentReader;
    
    /**
     * The lexer that tells us what colors different
     * words should be.
     */
    protected Lexer syntaxLexer;
    
    /**
     * A thread that handles the actual coloring.
     */
    protected Colorer colorer;
    
    /**
     * A lock for modifying the document, or for
     * actions that depend on the document not being
     * modified.
     */
    private Object doclock = new Object();

	/** Used for sizing windows */
	private Dimension screenSize;

	/** Window width */
	private int screenWidth;

	/** Window height */
	private int screenHeight;

	/** Pointer to current instance */
	final TextEditFrame tef;

	/** Pop up menu */
	private JPopupMenu popup;

	/** Pointer to current MainFrame */
	final private MainFrame currentParent;

	/** Temporary string */
	private String temp;
	
	/** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages"); //$NON-NLS-1$

	/**Current path */
	private File currentPath=new File(".");

	/** Constructor for new query/mod */

	public TextEditFrame(final Component ancestor,
			boolean type, String query) {
//		Load a SQLLexer 
		syntaxLexer = new SQLLexer(documentReader);
		//Create the document model.
        document = new HighLightedDocument();
        // Create the text pane and configure it.
        textArea = new JTextPane(document);
        textArea.setCaretPosition(0);
        textArea.setMargin(new Insets(2,2,2,2));
        JScrollPane scrollPane = new JScrollPane(textArea);
        // specify the initial size and location for the window.
        scrollPane.setPreferredSize(new Dimension(560, 300));
        setLocation(0, 0);
        // Add the components to the frame.
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.add(scrollPane, BorderLayout.CENTER);
        setContentPane(contentPane);
        // Start the thread that does the coloring
        colorer = new Colorer();
        colorer.start();
        // Set up the hash table that contains the styles.
        initStyles();
        // create the new document.
        documentReader = new DocumentReader(document);
        currentParent = (MainFrame) ancestor;
		popup = new JPopupMenu();
		JMenuItem cut = new JMenuItem(Messages.getString("TextEditFrame.0"), new ImageIcon("Resources/cut.gif")); //$NON-NLS-1$ //$NON-NLS-2$
		JMenuItem paste = new JMenuItem(Messages.getString("TextEditFrame.2"), new ImageIcon( //$NON-NLS-1$
				"Resources/paste.gif")); //$NON-NLS-1$
		JMenuItem copy = new JMenuItem(Messages.getString("TextEditFrame.6"), new ImageIcon( //$NON-NLS-1$
				"Resources/copy.gif")); //$NON-NLS-1$
		JMenuItem printb = new JMenuItem(Messages.getString("Table.0"), new ImageIcon( //$NON-NLS-1$
				"Resources/print.gif")); //$NON-NLS-1$
		//System.out.println(screenWidth+" "+screenHeight);
		popup.add(cut);
		popup.add(copy);
		popup.add(paste);
		popup.add(printb);

		cut.addActionListener(new Cut());
		copy.addActionListener(new Copy());
		paste.addActionListener(new Paste());
		printb.addActionListener(new Print());

		tef = this;

		setSize(screenWidth / 2, screenHeight / 2);
		typ = type;
		if (typ) {
			setTitle(currentParent.windowTitle);
			setLocation(currentParent.winpos , currentParent.winpos);
			currentParent.winpos=(currentParent.winpos)+10;
			setFrameIcon(new ImageIcon("Resources/palette_draw_query.gif")); //$NON-NLS-1$

		}
		if (!typ) {
			setTitle(currentParent.windowTitle);
			setLocation(currentParent.winpos +300, currentParent.winpos);
			currentParent.winpos=(currentParent.winpos)+10;
			setFrameIcon(new ImageIcon("Resources/palette_draw_mod.gif")); //$NON-NLS-1$
		}

		currentParent.winpos = currentParent.winpos + 30;
		if (currentParent.winpos >= 150)
			currentParent.winpos = 0;

		
		JMenuBar mb = new JMenuBar();
		JMenu jm = new JMenu(Messages.getString("TextEditFrame.10")); //$NON-NLS-1$
		mb.add(jm);
		jm.setMnemonic('a');
		JMenuItem ex = new JMenuItem(Messages.getString("TextEditFrame.11"), new ImageIcon( //$NON-NLS-1$
				"Resources/next.gif")); //$NON-NLS-1$
		JMenuItem canc = new JMenuItem(Messages.getString("PasswordChooser.3"), new ImageIcon( //$NON-NLS-1$
				"Resources/prev.gif")); //$NON-NLS-1$
		JMenuItem save = new JMenuItem(Messages.getString("Table.2"), new ImageIcon( //$NON-NLS-1$
				"Resources/save.gif")); //$NON-NLS-1$
		JMenuItem check = new JMenuItem(Messages.getString("TextEditFrame.17"), new ImageIcon( //$NON-NLS-1$
				"Resources/JEditorPane.gif")); //$NON-NLS-1$
		JMenuItem print = new JMenuItem(Messages.getString("Table.0"), new ImageIcon( //$NON-NLS-1$
				"Resources/print.gif")); //$NON-NLS-1$

		jm.add(check);
		jm.add(print);
		jm.add(ex);
		jm.add(save);
		jm.add(canc);

		setJMenuBar(mb);

		this.setClosable(true);
		this.setMaximizable(true);
		this.setIconifiable(true);
		this.setResizable(true);

//		desktop.add(this, 1);
		currentParent.frameDesktop.add(this);
		
		this.show();

		
		textArea.setDragEnabled(true);
		textArea.addMouseListener(new RightClick());

		ex.addActionListener((new Exec()));
		canc.addActionListener(new Canc());
		save.addActionListener(new Save());
		check.addActionListener(new Check());
		print.addActionListener(new Print());

		
		contentPane.add(scrollPane, BorderLayout.CENTER);
		
		if (query!=null)
		textArea.setText(query);
		
		textArea.setCaretPosition(0);
		textArea.setCaretColor(Color.RED);
//		put it all together and show it.
        pack();
        setVisible(true);
    	}

	    /**
     * retrieve the style for the given type of text.
     *
     * @param styleName the label for the type of text ("tag" for example) 
	 *      or null if the styleName is not known.
     * @return the style
     */
    private SimpleAttributeSet getStyle(String styleName){
        return ((SimpleAttributeSet)styles.get(styleName));
    }

    /**
     * Create the styles and place them in the hash table.
     */
    private void initStyles(){
        SimpleAttributeSet style;

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.black);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("body", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.blue);
        StyleConstants.setBold(style, true);
        StyleConstants.setItalic(style, false);
        styles.put("tag", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.blue);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("endtag", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.black);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("reference", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(0xB03060)/*Color.maroon*/);
        StyleConstants.setBold(style, true);
        StyleConstants.setItalic(style, false);
        styles.put("name", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(0xB03060)/*Color.maroon*/);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, true);
        styles.put("value", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.black);
        StyleConstants.setBold(style, true);
        StyleConstants.setItalic(style, false);
        styles.put("text", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.blue);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("reservedWord", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.DARK_GRAY);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("identifier", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(0xB03060)/*Color.maroon*/);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("literal", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(0x000080)/*Color.navy*/);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("separator", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(250,100,100));
        StyleConstants.setBold(style, true);
        StyleConstants.setItalic(style, false);
        styles.put("operator", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.green.darker());
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("comment", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, new Color(0xA020F0).darker()/*Color.purple*/);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("preprocessor", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.black);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("whitespace", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.red);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("error", style);

        style = new SimpleAttributeSet();
        StyleConstants.setFontFamily(style, "Monospaced");
        StyleConstants.setFontSize(style, 12);
        StyleConstants.setBackground(style, Color.white);
        StyleConstants.setForeground(style, Color.orange);
        StyleConstants.setBold(style, false);
        StyleConstants.setItalic(style, false);
        styles.put("unknown", style);
    }


	/** Popup menu management */
	class RightClick extends MouseAdapter {

		/** No action needed */
		public void mouseClicked(MouseEvent me) {

		}

		/** Right click */
		public void mousePressed(MouseEvent me) {
			//Invoked by Linux
			if (me.isPopupTrigger())
				popup.show(me.getComponent(), me.getX(), me.getY());

		}

		/** Right click */
		public void mouseReleased(MouseEvent me) {
			//Invoked by Windows
			if (me.isPopupTrigger())
				popup.show(me.getComponent(), me.getX(), me.getY());

		}

		/** No action needed */
		public void mouseEntered(MouseEvent me) {

		}

		/** No action needed */
		public void mouseExited(MouseEvent me) {

		}
	}

	/** Cut action */
	class Cut implements ActionListener {

		public void actionPerformed(ActionEvent ae) {
			textArea.cut();
		}
	}

	/** Copy action */
	class Copy implements ActionListener {
 
		public void actionPerformed(ActionEvent ae) {
			textArea.copy();
		}

	}

	/** Paste action */
	class Paste implements ActionListener {

		public void actionPerformed(ActionEvent ae) {
			textArea.paste();
		}

	}

	/** Print action */
	class Print implements ActionListener {

		public void actionPerformed(ActionEvent ae) {

			 PrintUtilities.printComponent(textArea);
			 
					}
	}
		/** Query mod execution */
	class Exec implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (typ) {

				String text = textArea.getText();
				BufferedResultSet prs = currentParent.currentAccess
						.executeQuery(text);
				if (prs != null) {
					currentParent.tab = new Table(prs.doc, prs.columnNames,
							currentParent.frameDesktop, Messages.getString("TextEditFrame.52"), prs.columnType, currentParent); //$NON-NLS-1$
					currentParent.currentQuery = text;
					currentParent.tab.show();

				}
			} else
				currentParent.currentAccess.executeUpdein(textArea.getText(),true);

		}
	}

	/** Cancel text */
	class Canc implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			try {
				document.remove(0,document.getLength());  //$NON-NLS-1$
			} catch (BadLocationException e) {
			
				e.printStackTrace();
			}
			textArea.setCaretColor(Color.RED);
			textArea.setCaretPosition(0);
		}
		/** Well form check */
	}

	/** Parse utility */
	class Check implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			MiniParser pars = new MiniParser();
			try {
				String text = document.getText(0,document.getLength());
				document.remove(0,document.getLength());
				document.insertString(0 ,(pars.reorder(text)), getStyle("text"));  //$NON-NLS-1$
			} catch (BadLocationException e) {
			
				e.printStackTrace();
			}
			textArea.setCaretPosition(0);
			textArea.setCaretColor(Color.BLUE);

		}
	}

	/** Save file */
	class Save implements ActionListener {

		public void actionPerformed(ActionEvent evt) {
			JFileChooser chooser = new JFileChooser();
			
			chooser.setCurrentDirectory(currentPath); //$NON-NLS-1$
			if (typ) {
				chooser.addChoosableFileFilter(new QueryFilter());
				chooser.setFileView(new FileIconViewForQuery(new QueryFilter(),
						new ImageIcon("Resources/query.gif"))); //$NON-NLS-1$
			} else {
				chooser.addChoosableFileFilter(new ModFilter());
				chooser.setFileView(new FileIconViewForMod(new ModFilter(),
						new ImageIcon("Resources/mod.gif"))); //$NON-NLS-1$
			}

			int r = chooser.showSaveDialog(null);

			if (r == JFileChooser.APPROVE_OPTION) {
				currentPath=chooser.getSelectedFile();
				currentParent.windowTitle = chooser.getSelectedFile().getName();
				temp = chooser.getSelectedFile().getAbsolutePath();
				if ((typ) && (!temp.endsWith(".qry"))) { //$NON-NLS-1$
					temp = temp.concat(".qry"); //$NON-NLS-1$
					currentParent.windowTitle = currentParent.windowTitle
							.concat(".qry"); //$NON-NLS-1$
				}

				if (!typ) {
					if (!temp.endsWith(".mod")) { //$NON-NLS-1$
						temp = temp.concat(".mod"); //$NON-NLS-1$
						currentParent.windowTitle = currentParent.windowTitle
								.concat(".mod"); //$NON-NLS-1$
					}
				}

				/*	if ((new FileManager()).saveFile(textArea.getText(), temp))
					JOptionPane.showMessageDialog(null, Messages.getString("Table.17") //$NON-NLS-1$
							+ currentParent.windowTitle, Messages.getString("MainFrame.Information"), 1); //$NON-NLS-1$
				else
					JOptionPane.showMessageDialog(null,
							Messages.getString("TextEditFrame.65"), Messages.getString("MainFrame.Information"), 0); //$NON-NLS-1$ //$NON-NLS-2$
							*/				
				currentPath=new File(temp);
				if ((new FileManager()).saveFile(textArea.getText(), temp))
				tef.setTitle(currentParent.windowTitle);
			}

		}
	}

	/**
	 * A reader interface for an abstract document.  Since
	 * the syntax highlighting packages only accept Stings and
	 * Readers, this must be used.
	 * Since the close() method does nothing and a seek() method
	 * has been added, this allows us to get some performance
	 * improvements through reuse.  It can be used even after the
	 * lexer explicitly closes it by seeking to the place that
	 * we want to read next, and reseting the lexer.
	 */

	
	class DocumentReader extends Reader {

	    /**
	     * Modifying the document while the reader is working is like
	     * pulling the rug out from under the reader.  Alerting the
	     * reader with this method (in a nice thread safe way, this
	     * should not be called at the same time as a read) allows
	     * the reader to compensate.
	     */
	    public void update(int position, int adjustment){
	        if (position < this.position){
	            if (this.position < position - adjustment){
	                this.position = position;
	            } else {
	                this.position += adjustment;
	            }
	        }
	    }

	    /**
	     * Current position in the document. Incremented
	     * whenever a character is read.
	     */
	    private long position = 0;

	    /**
	     * Saved position used in the mark and reset methods.
	     */
	    private long mark = -1;

	    /**
	     * The document that we are working with.
	     */
	    private AbstractDocument document;

	    /**
	     * Construct a reader on the given document.
	     *
	     * @param document the document to be read.
	     */
	    public DocumentReader(AbstractDocument document){
	        this.document = document;
	    }

	    /**
	     * Has no effect.  This reader can be used even after
	     * it has been closed.
	     */
	    public void close() {
	    }

	    /**
	     * Save a position for reset.
	     *
	     * @param readAheadLimit ignored.
	     */
	    public void mark(int readAheadLimit){
	        mark = position;
	    }

	    /**
	     * This reader support mark and reset.
	     *
	     * @return true
	     */
	    public boolean markSupported(){
	        return true;
	    }

	    /**
	     * Read a single character.
	     *
	     * @return the character or -1 if the end of the document has been reached.
	     */
	    public int read(){
	        if (position < document.getLength()){
	            try {
	                char c = document.getText((int)position, 1).charAt(0);
	                position++;
	                return c;
	            } catch (BadLocationException x){
	                return -1;
	            }
	        } else {
	            return -1;
	        }
	    }

	    /**
	     * Read and fill the buffer.
	     * This method will always fill the buffer unless the end of the document is reached.
	     *
	     * @param cbuf the buffer to fill.
	     * @return the number of characters read or -1 if no more characters are available in the document.
	     */
	    public int read(char[] cbuf){
	        return read(cbuf, 0, cbuf.length);
	    }

	    /**
	     * Read and fill the buffer.
	     * This method will always fill the buffer unless the end of the document is reached.
	     *
	     * @param cbuf the buffer to fill.
	     * @param off offset into the buffer to begin the fill.
	     * @param len maximum number of characters to put in the buffer.
	     * @return the number of characters read or -1 if no more characters are available in the document.
	     */
	    public int read(char[] cbuf, int off, int len){
	        if (position < document.getLength()){
	            int length = len;
	            if (position + length >= document.getLength()){
	                length = document.getLength() - (int)position;
	            }
	            if (off + length >= cbuf.length){
	                length = cbuf.length - off;
	            }
	            try {
	                String s = document.getText((int)position, length);
	                position += length;
	                for (int i=0; i<length; i++){
	                    cbuf[off+i] = s.charAt(i);
	                }
	                return length;
	            } catch (BadLocationException x){
	                return -1;
	            }
	        } else {
	            return -1;
	        }
	    }

	    /**
	     * @return true
	     */
	    public boolean ready() {
	        return true;
	    }

	    /**
	     * Reset this reader to the last mark, or the beginning of the document if a mark has not been set.
	     */
	    public void reset(){
	        if (mark == -1){
	            position = 0;
	        } else {
	            position = mark;
	        }
	        mark = -1;
	    }

	    /**
	     * Skip characters of input.
	     * This method will always skip the maximum number of characters unless
	     * the end of the file is reached.
	     *
	     * @param n number of characters to skip.
	     * @return the actual number of characters skipped.
	     */
	    public long skip(long n){
	        if (position + n <= document.getLength()){
	            position += n;
	            return n;
	        } else {
	            long oldPos = position;
	            position = document.getLength();
	            return (document.getLength() - oldPos);
	        }
	    }

	    /**
	     * Seek to the given position in the document.
	     *
	     * @param n the offset to which to seek.
	     */
	    public void seek(long n){
	        if (n <= document.getLength()){
	            position = n;
	        } else {
	            position = document.getLength();
	        }
	    }
	}
	/**
	 * A comparator appropriate for use with Collections of
	 * DocPositions.
	 */
	class DocPositionComparator implements Comparator{
	    /**
	     * Does this Comparator equal another?
	     * Since all DocPositionComparators are the same, they
	     * are all equal.
	     *
	     * @return true for DocPositionComparators, false otherwise.
	     */
	    public boolean equals(Object obj){
	        if (obj instanceof DocPositionComparator){
	            return true;
	        } else {
	            return false;
	        }
	    }

	    /**
	     * Compare two DocPositions
	     *
	     * @param o1 first DocPosition
	     * @param o2 second DocPosition
	     * @return negative if first < second, 0 if equal, positive if first > second
	     */
	    public int compare(Object o1, Object o2){
	        if (o1 instanceof DocPosition && o2 instanceof DocPosition){
	            DocPosition d1 = (DocPosition)(o1);
	            DocPosition d2 = (DocPosition)(o2);
	            return (d1.getPosition() - d2.getPosition());
	        } else if (o1 instanceof DocPosition){
	            return -1;
	        } else if (o2 instanceof DocPosition){
	            return 1;
	        } else if (o1.hashCode() < o2.hashCode()){
	            return -1;
	        } else if (o2.hashCode() > o1.hashCode()){
	            return 1;
	        } else {
	            return 0;
	        }
	    }
	}
	/**
	 * A wrapper for a position in a document appropriate for storing
	 * in a collection.
	 */
	class DocPosition {

	    /**
	     * The actual position
	     */
	    private int position;

	    /**
	     * Get the position represented by this DocPosition
	     *
	     * @return the position
	     */
	    int getPosition(){
	        return position;
	    }

	    /**
	     * Construct a DocPosition from the given offset into the document.
	     *
	     * @param position The position this DocObject will represent
	     */
	    public DocPosition(int position){
	        this.position = position;
	    }

	    /**
	     * Adjust this position.
	     * This is useful in cases that an amount of text is inserted
	     * or removed before this position.
	     *
	     * @param adjustment amount (either positive or negative) to adjust this position.
	     * @return the DocPosition, adjusted properly.
	     */
	    public DocPosition adjustPosition(int adjustment){
	        position += adjustment;
	        return this;
	    }

	    /**
	     * Two DocPositions are equal iff they have the same internal position.
	     *
	     * @return if this DocPosition represents the same position as another.
	     */
	    public boolean equals(Object obj){
	        if (obj instanceof DocPosition){
	            DocPosition d = (DocPosition)(obj);
	            if (this.position == d.position){
	                return true;
	            } else {
	                return false;
	            }
	        } else {
	            return false;
	        }
	    }

	    /**
	     * A string representation useful for debugging.
	     *
	     * @return A string representing the position.
	     */
	    public String toString(){
	        return "" + position;
	    }
	}
    /**
     * Just like a DefaultStyledDocument but intercepts inserts and
     * removes to color them.
     */
    private class HighLightedDocument extends DefaultStyledDocument {
        public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
            synchronized (doclock){
                super.insertString(offs, str, a);
                color(offs, str.length());
                documentReader.update(offs, str.length());
            }
        }

        public void remove(int offs, int len) throws BadLocationException {
            synchronized (doclock){
                super.remove(offs, len);
                color(offs, -len);
                documentReader.update(offs, -len);
            }
        }
    }
    /**
     * Run the Syntax Highlighting as a separate thread.
     * Things that need to be colored are messaged to the
     * thread and put in a list.
     */
    private class Colorer extends Thread {

        /**
         * Keep a list of places in the file that it is safe to restart the
         * highlighting.  This happens whenever the lexer reports that it has
         * returned to its initial state.  Since this list needs to be sorted
         * and we need to be able to retrieve ranges from it, it is stored in a
         * balanced tree.
         */
        private TreeSet iniPositions = new TreeSet(new DocPositionComparator());

        /**
         * As we go through and remove invalid positions we will also be finding
		 * new valid positions. 
		 * Since the position list cannot be deleted from and written to at the same
         * time, we will keep a list of the new positions and simply add it to the
         * list of positions once all the old positions have been removed.
         */
    	private HashSet newPositions = new HashSet();

        /**
         * A simple wrapper representing something that needs to be colored.
         * Placed into an object so that it can be stored in a Vector.
         */
        private class RecolorEvent {
            public int position;
            public int adjustment;
            public RecolorEvent(int position, int adjustment){
                this.position = position;
                this.adjustment = adjustment;
            }
        }

        /**
         * Vector that stores the communication between the two threads.
         */
        private volatile Vector v = new Vector();

        /**
         * The amount of change that has occurred before the place in the
         * document that we are currently highlighting (lastPosition).
         */
        private volatile int change = 0;

        /**
         * The last position colored
         */
        private volatile int lastPosition = -1;
        
        private volatile boolean asleep = false;

        /**
         * When accessing the vector, we need to create a critical section.
         * we will synchronize on this object to ensure that we don't get
         * unsafe thread behavior.
         */
        private Object lock = new Object();

        /**
         * Tell the Syntax Highlighting thread to take another look at this
         * section of the document.  It will process this as a FIFO.
         * This method should be done inside a doclock.
         */
        public void color(int position, int adjustment){
            // figure out if this adjustment effects the current run.
            // if it does, then adjust the place in the document
            // that gets highlighted.
            if (position < lastPosition){
                if (lastPosition < position - adjustment){
                    change -= lastPosition - position;
				} else {
                	change += adjustment;
                }
            }
            synchronized(lock){
                v.add(new RecolorEvent(position, adjustment));
                if (asleep){
                    this.interrupt();
                }
            }
        }

        /**
         * The colorer runs forever and may sleep for long
         * periods of time.  It should be interrupted every
         * time there is something for it to do.
         */
        public void run(){
            int position = -1;
            int adjustment = 0;
            // if we just finish, we can't go to sleep until we
            // ensure there is nothing else for us to do.
            // use try again to keep track of this.
            boolean tryAgain = false;
            for (;;){  // forever
                synchronized(lock){
                    if (v.size() > 0){
                        RecolorEvent re = (RecolorEvent)(v.elementAt(0));
                        v.removeElementAt(0);
                        position = re.position;
                        adjustment = re.adjustment;
                    } else {
                        tryAgain = false;
                        position = -1;
                        adjustment = 0;
                    }
                }
                if (position != -1){
                    SortedSet workingSet;
                    Iterator workingIt;
                    DocPosition startRequest = new DocPosition(position);
                    DocPosition endRequest = new DocPosition(position + ((adjustment>=0)?adjustment:-adjustment));
                    DocPosition  dp;
                    DocPosition dpStart = null;
                    DocPosition dpEnd = null;

                    // find the starting position.  We must start at least one
                    // token before the current position
                    try {
                        // all the good positions before
                        workingSet = iniPositions.headSet(startRequest);
                        // the last of the stuff before
                        dpStart = ((DocPosition)workingSet.last());
                    } catch (NoSuchElementException x){
                        // if there were no good positions before the requested start,
                        // we can always start at the very beginning.
                        dpStart = new DocPosition(0);
                    }

                    // if stuff was removed, take any removed positions off the list.
                    if (adjustment < 0){
                        workingSet = iniPositions.subSet(startRequest, endRequest);
                        workingIt = workingSet.iterator();
                        while (workingIt.hasNext()){
                            workingIt.next();
                            workingIt.remove();
                        }
                    }

                    // adjust the positions of everything after the insertion/removal.
                    workingSet = iniPositions.tailSet(startRequest);
                    workingIt = workingSet.iterator();
                    while (workingIt.hasNext()){
                        ((DocPosition)workingIt.next()).adjustPosition(adjustment);
                    }

                    // now go through and highlight as much as needed
                    workingSet = iniPositions.tailSet(dpStart);
                    workingIt = workingSet.iterator();
                    dp = null;
                    if (workingIt.hasNext()){
                        dp = (DocPosition)workingIt.next();
                    }
                    try {
                        Token t;
                        boolean done = false;
                        dpEnd = dpStart;
                        synchronized (doclock){
                            // we are playing some games with the lexer for efficiency.
                            // we could just create a new lexer each time here, but instead,
                            // we will just reset it so that it thinks it is starting at the
                            // beginning of the document but reporting a funny start position.
                            // Reseting the lexer causes the close() method on the reader
                            // to be called but because the close() method has no effect on the
                            // DocumentReader, we can do this.
                            syntaxLexer.reset(documentReader, 0, dpStart.getPosition(), 0);
                            // After the lexer has been set up, scroll the reader so that it
                            // is in the correct spot as well.
                            documentReader.seek(dpStart.getPosition());
                            // we will highlight tokens until we reach a good stopping place.
                            // the first obvious stopping place is the end of the document.
                            // the lexer will return null at the end of the document and wee
                            // need to stop there.
                            t = syntaxLexer.getNextToken();
                        }
                        newPositions.add(dpStart);
                        while (!done && t != null){
                            // this is the actual command that colors the stuff.
                            // Color stuff with the description of the style matched
                            // to the hash table that has been set up ahead of time.
                            synchronized (doclock){
                                if (t.getCharEnd() <= document.getLength()){
                                    document.setCharacterAttributes(
                                        t.getCharBegin() + change,
                                        t.getCharEnd()-t.getCharBegin(),
                                        getStyle(t.getDescription()),
                                        true
                                    );                                                                  
                                    // record the position of the last bit of text that we colored
                                    dpEnd = new DocPosition(t.getCharEnd());
                                }
                                lastPosition = (t.getCharEnd() + change);
                            }
                            // The other more complicated reason for doing no more highlighting
                            // is that all the colors are the same from here on out anyway.
                            // We can detect this by seeing if the place that the lexer returned
                            // to the initial state last time we highlighted is the same as the
                            // place that returned to the initial state this time.
                            // As long as that place is after the last changed text, everything
                            // from there on is fine already.
                            if (t.getState() == Token.INITIAL_STATE){
                                //System.out.println(t);
                                // look at all the positions from last time that are less than or
                                // equal to the current position
                                while (dp != null && dp.getPosition() <= t.getCharEnd()){
                                    if (dp.getPosition() == t.getCharEnd() && dp.getPosition() >= endRequest.getPosition()){
                                        // we have found a state that is the same
										done = true;
                                        dp = null;
                                    } else if (workingIt.hasNext()){
                                        // didn't find it, try again.
                                        dp = (DocPosition)workingIt.next();
                                    } else {
                                        // didn't find it, and there is no more info from last
                                        // time.  This means that we will just continue
                                        // until the end of the document.
                                        dp = null;
                                    }
                                }
                                // so that we can do this check next time, record all the
                                // initial states from this time.
                                newPositions.add(dpEnd);
                            }
                            synchronized (doclock){
                                t = syntaxLexer.getNextToken();
                            }
                        }

                        // remove all the old initial positions from the place where
                        // we started doing the highlighting right up through the last
                        // bit of text we touched.
                        workingIt = iniPositions.subSet(dpStart, dpEnd).iterator();
                        while (workingIt.hasNext()){
                            workingIt.next();
                            workingIt.remove();
                        }
                        
                        // Remove all the positions that are after the end of the file.:
                        workingIt = iniPositions.tailSet(new DocPosition(document.getLength())).iterator();
                        while (workingIt.hasNext()){
                            workingIt.next();
                            workingIt.remove();
                        }

                        // and put the new initial positions that we have found on the list.
                        iniPositions.addAll(newPositions);
                        newPositions.clear();
                        
                        /*workingIt = iniPositions.iterator();
                        while (workingIt.hasNext()){
                            System.out.println(workingIt.next());
                        }
                        
                        System.out.println("Started: " + dpStart.getPosition() + " Ended: " + dpEnd.getPosition());*/
                    } catch (IOException x){
                    }
                    synchronized (doclock){
                        lastPosition = -1;
                        change = 0;
                    }
                    // since we did something, we should check that there is
                    // nothing else to do before going back to sleep.
                    tryAgain = true;
                }                
                asleep = true;
                if (!tryAgain){
                    try {
                        sleep (0xffffff);
                    } catch (InterruptedException x){
                    }
                    
                }
                asleep = false;
            }
        }
    
    }
    /**
     * Color or recolor the entire document
     */
    public void colorAll(){
        color(0, document.getLength());
    }
    /**
     * Color a section of the document.
     * The actual coloring will start somewhere before
     * the requested position and continue as long
     * as needed.
     *
     * @param position the starting point for the coloring.
     * @param adjustment amount of text inserted or removed
     *    at the starting point.
     */
    public void color(int position, int adjustment){
        colorer.color(position, adjustment);
    }

    
}