/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package tools;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.swing.JDesktopPane;
import javax.swing.JEditorPane;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

/** Class that manages the help on line window */
public class HtmlHelpOnLine extends JInternalFrame {

    /** Place for html page */
    JEditorPane html;
    
    /** Resource bundle for localization */
	private ResourceBundle Messages=ResourceBundle.getBundle("Lang/messages");
	
    /** Main constructor */
    public HtmlHelpOnLine(JDesktopPane desktop) {

        java.net.URL url = null;
        String path = null;

        try {

            Properties prop = System.getProperties();
            path=(prop.getProperty("user.dir")).toString(); //$NON-NLS-1$
            //path=path.substring(1,path.length());
            path="file:"+path+"/Docs/HelpOnLine/Index.htm"; //$NON-NLS-1$ //$NON-NLS-2$
    		path=path.replace('\\','/');
            System.out.println("Documentation path is\n "+path); //$NON-NLS-1$
            url = new java.net.URL(path);
            // System.out.println(url);
            html = new JEditorPane(url);
            html.setEditable(false);
            html.addHyperlinkListener(createHyperLinkListener());
            Properties p=System.getProperties();
            p.list(System.out);
            JScrollPane scroller = new JScrollPane();
            JViewport vp = scroller.getViewport();
            vp.add(html);

            setTitle("Help On Line"); //$NON-NLS-1$
            
            setLocation(0, 0);

            Toolkit kit = Toolkit.getDefaultToolkit();
            Dimension screenSize = kit.getScreenSize();
            int screenWidth = ((screenSize.width) - 150);
            int screenHeight = ((screenSize.height) - 300);
            setSize(screenWidth, screenHeight);

            Container contentPane = getContentPane();
            contentPane.add(scroller, BorderLayout.CENTER);

            this.setClosable(true);
            this.setMaximizable(true);
            this.setIconifiable(true);
            this.setResizable(true);

            desktop.add(this, 1);
            this.show();

        } catch (MalformedURLException e1) {
            JOptionPane.showMessageDialog(null, "Malformed URL Exception ! ", //$NON-NLS-1$
                    Messages.getString("MainFrame.Information"), 0); //$NON-NLS-1$
        } catch (IOException e) {

            JOptionPane.showMessageDialog(null, Messages.getString("HtmlHelpOnLine.7"), Messages.getString("MainFrame.Information"), 2); //$NON-NLS-1$ //$NON-NLS-2$

        }
    }

    /** This allows using hiperlink on help on line page */
    public HyperlinkListener createHyperLinkListener() {
        return new HyperlinkListener() {

            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    if (e instanceof HTMLFrameHyperlinkEvent) {
                        ((HTMLDocument) html.getDocument())
                                .processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent) e);
                    } else {
                        try {
                            html.setPage(e.getURL());
                        } catch (IOException ioe) {
                            JOptionPane.showMessageDialog(null,
                                    "I/O exception", Messages.getString("MainFrame.Information"), 0); //$NON-NLS-1$ //$NON-NLS-2$
                        }
                    }
                }
            }
        };
    }

}