/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package tools;

import java.util.StringTokenizer;

/**
 * This class provides some useful methods to manipulate a query text
 */
public class MiniParser {

	/** Main constructor */
	public MiniParser() {

	}

	/** This procedure well forms the text
	 @version 0.4
	 **/
	public String reorder(String text) {
		String reorganized = new String();
		String temp = null;
		//String space = "                        ";
		String space = "                    ";
		int nestingLevel = 0;
		boolean subqueryFound = false;
		boolean perahpsSubqueryEnded = false;
		boolean thereWasDelete = false;
		boolean triggerDetected = false;
		boolean perhapsQueryStarted = false;
		boolean itWasEnd = false;
		boolean checkNewLine = false;
		boolean itWasUEI = false;
		boolean firstChar = true;
		boolean isInTable = false;
		boolean itWasInTable = false;
		boolean checkDetected = false;
		//boolean isInComment =false;
		StringTokenizer st = new StringTokenizer(text);
		
		while (st.hasMoreTokens() /* && nestingLevel>=0 */) {
			temp = st.nextToken();
			//This is neccessary for compatibility between the diffrent
			// keywords
			if (nestingLevel < 0)
				nestingLevel = 0;
			// System.out.println(checkDetected);
			// System.out.println(""+isInTable);

			if (temp.equalsIgnoreCase("SELECT")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				if (subqueryFound) {
					reorganized = reorganized
							.concat(" SELECT "/* +nestingLevel */);
					nestingLevel++;
					subqueryFound = false;
					perhapsQueryStarted = false;
				} 
				/*if (isInComment) {
					isInComment=false;
					reorganized=reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized.concat("SELECT ");

				}*/
				else if (perahpsSubqueryEnded) {
					nestingLevel--;
					perahpsSubqueryEnded = false;
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized
							.concat("SELECT "/* +nestingLevel */);
				} else {
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized
							.concat("SELECT"/* +nestingLevel */);
				}

			}
			
			else if (temp.equalsIgnoreCase("(SELECT")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				reorganized = reorganized.concat(" (SELECT ");
				nestingLevel++;
				subqueryFound = false;
			}

			else if (temp.equalsIgnoreCase("(SELECT*")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				reorganized = reorganized.concat(" ( SELECT *");
				nestingLevel++;
				subqueryFound = false;
			}

			else if (temp.endsWith("=(SELECT") || temp.endsWith(",(SELECT")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				reorganized = reorganized.concat(" " + temp + " ");
				nestingLevel++;
				subqueryFound = false;
			}

			else if (temp.endsWith("=(select")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				int l = (temp.length() - 8);
				reorganized = reorganized.concat(" " + temp.substring(0, l)
						+ " = ( SELECT ");
				nestingLevel++;
				subqueryFound = false;
			}

			else if (temp.endsWith(",(select")) {
				if (isInTable) {
					isInTable = false;
					itWasInTable = true;
					checkDetected = false;
				}
				int l = (temp.length() - 8);
				reorganized = reorganized.concat(" " + temp.substring(0, l)
						+ " , ( SELECT ");
				nestingLevel++;
				subqueryFound = false;
			}

			else if (temp.equalsIgnoreCase("FROM")) {
				if (!thereWasDelete) {
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized.concat("FROM");
				} else {
					reorganized = reorganized.concat(" FROM ");
					thereWasDelete = false;
				}
			}
			else if (temp.startsWith("--"))
			{
				
				reorganized= reorganized.concat("\n"+temp);
				
			}

			else if (temp.equalsIgnoreCase("WHERE")) {
				if (perahpsSubqueryEnded) {
					nestingLevel--;
					perahpsSubqueryEnded = false;
				}
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("WHERE ");
			}

			else if (temp.equalsIgnoreCase("GROUP")) {
				if (perahpsSubqueryEnded) {
					nestingLevel--;
					perahpsSubqueryEnded = false;
				}
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("GROUP");
			}

			else if (temp.equalsIgnoreCase("ORDER")) {
				if (perahpsSubqueryEnded) {
					nestingLevel--;
					perahpsSubqueryEnded = false;
				}
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("ORDER");
			}

			else if (temp.equalsIgnoreCase("UPDATE")) {
				isInTable = false;
				if (!triggerDetected) {
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized.concat("UPDATE ");
				} else {
					reorganized = reorganized.concat(" UPDATE ");
					triggerDetected = false;
				}
			}

			else if (temp.equalsIgnoreCase("DELETE")) {
				isInTable = false;
				if (!triggerDetected) {
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					reorganized = reorganized.concat("DELETE ");
					thereWasDelete = true;
				} else {
					reorganized = reorganized.concat(" DELETE ");
					triggerDetected = false;
				}
			}

			else if (temp.equalsIgnoreCase("SET")) {
				isInTable = false;
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("SET ");
			}

			else if (temp.equalsIgnoreCase("union")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("UNION");
				itWasUEI = true;
			}

			else if (temp.equalsIgnoreCase("intersect")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("INTERSECT");
				itWasUEI = true;
			}

			else if (temp.equalsIgnoreCase("except")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("EXCEPT");
				itWasUEI = true;
			}

			else if (temp.startsWith("WHEN") || temp.startsWith("when")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("WHEN"
						+ temp.substring(4, temp.length()));
			}

			else if (temp.equalsIgnoreCase("INSERT")) {
				isInTable = false;
				if (!triggerDetected) {
					reorganized = reorganized.concat("\n");
					for (int i = 0; i < nestingLevel; i++)
						reorganized = reorganized.concat(space);
					
					reorganized = reorganized.concat("\nINSERT");
				} else {
					reorganized = reorganized.concat(" INSERT ");
					triggerDetected = false;
					nestingLevel = 0;
				}
			}

			else if (temp.equalsIgnoreCase("VALUES")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("VALUES");
			}

			else if (temp.equalsIgnoreCase("CREATE")) {
				isInTable = false;
				reorganized = reorganized.concat("\n\n");
				reorganized = reorganized.concat("CREATE ");
				nestingLevel = 0;
				triggerDetected = false;
				subqueryFound = false;
				perahpsSubqueryEnded = false;
			}

			else if (temp.equalsIgnoreCase("AFTER")) {
				reorganized = reorganized.concat("\n");
				reorganized = reorganized.concat("AFTER ");
				triggerDetected = true;
			}

			else if (temp.equalsIgnoreCase("BEFORE")) {
				reorganized = reorganized.concat("\n");
				reorganized = reorganized.concat("BEFORE");
				triggerDetected = true;
			}

			else if (temp.equalsIgnoreCase("DROP")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("DROP");
			}

			else if (temp.equalsIgnoreCase("ADD")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("ADD");
			}

			else if (temp.equalsIgnoreCase("having")) {
				reorganized = reorganized.concat("\n");
				for (int i = 0; i < nestingLevel; i++)
					reorganized = reorganized.concat(space);
				reorganized = reorganized.concat("HAVING");
			}

			else if (temp.equalsIgnoreCase("table")) {
				reorganized = reorganized.concat(" TABLE");
				isInTable = true;
			}

			else if (temp.equalsIgnoreCase("assertion")) {
				reorganized = reorganized.concat(" ASSERTION");
				checkNewLine = true;
			}

			else if (temp.equalsIgnoreCase("BY"))
				reorganized = reorganized.concat(" BY");

			else if (temp.equalsIgnoreCase("REFERENCING"))
				reorganized = reorganized.concat("\nREFERENCING");

			else if (temp.equalsIgnoreCase("on"))
				reorganized = reorganized.concat(" ON");

			else if (temp.equalsIgnoreCase("new"))
				reorganized = reorganized.concat("\nNEW");

			else if (temp.equalsIgnoreCase("old"))
				reorganized = reorganized.concat("\nOLD");

			else if (temp.equalsIgnoreCase("into"))
				reorganized = reorganized.concat(" INTO ");

			else if (temp.equalsIgnoreCase("for"))
				reorganized = reorganized.concat("\nFOR ");

			else if (temp.equalsIgnoreCase("each"))
				reorganized = reorganized.concat(" EACH ");

			else if (temp.equalsIgnoreCase("begin")) {
				reorganized = reorganized.concat("\nBEGIN ");
				nestingLevel = 0;
				subqueryFound = false;
				perhapsQueryStarted = false;
			}

			else if (temp.startsWith("end") || temp.startsWith("END")) {
				reorganized = reorganized.concat("\nEND "
						+ temp.substring(3, temp.length()) + " ");
				nestingLevel = 0;
				subqueryFound = false;
				perhapsQueryStarted = false;
				itWasEnd = true;
			}

			else if (temp.equalsIgnoreCase("row"))
				reorganized = reorganized.concat(" ROW");

			else if (temp.equalsIgnoreCase("on"))
				reorganized = reorganized.concat(" ON");

			else if (temp.equalsIgnoreCase("of"))
				reorganized = reorganized.concat(" OF");

			else if (temp.equalsIgnoreCase("out"))
				reorganized = reorganized.concat(" OUT");

			else if (temp.equalsIgnoreCase("inout"))
				reorganized = reorganized.concat(" INOUT");

			else if (temp.equalsIgnoreCase("date"))
				reorganized = reorganized.concat(" DATE");

			else if (temp.equalsIgnoreCase("time"))
				reorganized = reorganized.concat(" TIME");

			else if (temp.equalsIgnoreCase("unique"))
				reorganized = reorganized.concat(" UNIQUE");

			else if (temp.equalsIgnoreCase("in")) {
				reorganized = reorganized.concat(" IN");
				perhapsQueryStarted = true;
			}

			else if (temp.equalsIgnoreCase("join"))
				reorganized = reorganized.concat(" JOIN");

			else if (temp.equalsIgnoreCase("check")) {
				if (!checkNewLine) {
					reorganized = reorganized.concat(" CHECK");
					checkDetected = true;
				} else {
					checkNewLine = false;
					reorganized = reorganized.concat("\nCHECK");
				}
			}

			else if (temp.equalsIgnoreCase("exists"))
				reorganized = reorganized.concat(" EXISTS");

			else if (temp.equalsIgnoreCase("natural"))
				reorganized = reorganized.concat(" NATURAL");

			else if (temp.equalsIgnoreCase("cross"))
				reorganized = reorganized.concat(" CROSS");

			else if (temp.equalsIgnoreCase("outer"))
				reorganized = reorganized.concat(" OUTER");

			else if (temp.endsWith("all") || temp.endsWith("ALL"))
				reorganized = reorganized.concat(temp.substring(0, temp
						.length() - 3)
						+ " ALL");

			else if (temp.endsWith("any") || temp.endsWith("ANY"))
				reorganized = reorganized.concat(temp.substring(0, temp
						.length() - 3)
						+ " ANY");

			else if (temp.equalsIgnoreCase("as")) {
				reorganized = reorganized.concat(" AS ");
				if (perahpsSubqueryEnded) {
					nestingLevel--;
					perahpsSubqueryEnded = false;
					//reorganized=reorganized.concat(" <IO> ");
				}
			}

			else if (temp.equalsIgnoreCase("and"))
				reorganized = reorganized.concat(" AND");

			else if (temp.equalsIgnoreCase("distinct"))
				reorganized = reorganized.concat(" DISTINCT");

			//Put like startsWith all agregations
			else if (temp.equalsIgnoreCase("sum"))
				reorganized = reorganized.concat(" SUM");

			else if (temp.equalsIgnoreCase("sum(*)"))
				reorganized = reorganized.concat(" SUM(*)");

			else if (temp.equalsIgnoreCase("avg"))
				reorganized = reorganized.concat(" AVG");

			else if (temp.equalsIgnoreCase("avg(*)"))
				reorganized = reorganized.concat(" AVG(*)");

			else if (temp.equalsIgnoreCase("count"))
				reorganized = reorganized.concat(" COUNT");

			else if (temp.equalsIgnoreCase("count(*)"))
				reorganized = reorganized.concat(" COUNT(*)");

			else if (temp.equalsIgnoreCase("min"))
				reorganized = reorganized.concat(" MIN");

			else if (temp.equalsIgnoreCase("min(*)"))
				reorganized = reorganized.concat(" MIN(*)");

			else if (temp.equalsIgnoreCase("max"))
				reorganized = reorganized.concat(" MAX");

			else if (temp.equalsIgnoreCase("max(*)"))
				reorganized = reorganized.concat(" MAX(*)");

			else if (temp.equalsIgnoreCase("left"))
				reorganized = reorganized.concat(" LEFT");

			else if (temp.equalsIgnoreCase("right"))
				reorganized = reorganized.concat(" RIGHT");

			else if (temp.equalsIgnoreCase("full"))
				reorganized = reorganized.concat(" FULL");

			else if (temp.equalsIgnoreCase("cascade"))
				reorganized = reorganized.concat(" CASCADE");

			else if (temp.equalsIgnoreCase("null"))
				reorganized = reorganized.concat(" NULL");

			else if (temp.equalsIgnoreCase("default"))
				reorganized = reorganized.concat(" DEFAULT");

			else if (temp.equalsIgnoreCase("or"))
				reorganized = reorganized.concat(" OR");

			else if (temp.equalsIgnoreCase("not"))
				reorganized = reorganized.concat(" NOT");

			else if (temp.equalsIgnoreCase("like"))
				reorganized = reorganized.concat(" LIKE");

			else if (temp.startsWith("char") || temp.startsWith("CHAR"))
				reorganized = reorganized.concat(" CHAR"
						+ temp.substring(4, temp.length()));

			else if (temp.startsWith("varchar") || temp.startsWith("VARCHAR"))
				reorganized = reorganized.concat(" VARCHAR"
						+ temp.substring(7, temp.length()));

			else if (temp.equalsIgnoreCase("integer"))
				reorganized = reorganized.concat(" INTEGER");

			else if (temp.equalsIgnoreCase("float"))
				reorganized = reorganized.concat(" FLOAT");

			else if (temp.equalsIgnoreCase("trigger"))
			{
				reorganized = reorganized.concat(" TRIGGER");
				//triggerDetected=true;
			}
				

			else if (temp.equalsIgnoreCase("(")) {
				if (!firstChar) {
					if (!itWasUEI) {
						reorganized = reorganized.concat(" ( ");
						subqueryFound = true;
						if (perhapsQueryStarted)
							subqueryFound = true;
						//reorganized=reorganized.concat((new
						// Boolean(subqueryFound)).toString());
					} else {
						reorganized = reorganized.concat("\n");
						for (int i = 0; i < nestingLevel; i++)
							reorganized = reorganized.concat(space);
						reorganized = reorganized.concat(" ( ");
						itWasUEI = false;
						subqueryFound = false;
						perahpsSubqueryEnded = false;
					}
				} else
					reorganized = reorganized.concat("\n ( ");
			}

			else if (temp.endsWith(")WHERE") || temp.endsWith(")where")) {
				reorganized = reorganized.concat(" "
						+ temp.substring(0, temp.length() - 6) + " ) ");
				nestingLevel--;
			}

			else if (temp.endsWith(")")) {
				if (!itWasEnd) {
					if (itWasInTable) {
						itWasInTable = false;
						isInTable = true;
					}

					//reorganized=reorganized.concat("<" + nestingLevel+ ">" );
					reorganized = reorganized.concat(" "
							+ temp.substring(0, temp.length() - 1) + " ) ");
					if ((perahpsSubqueryEnded) && (nestingLevel > 0)) {
						nestingLevel--;
					}
					perahpsSubqueryEnded = true;
					subqueryFound = false;
				} else {
					reorganized = reorganized.concat(" "
							+ temp.substring(0, temp.length() - 1) + " ) ");
					perahpsSubqueryEnded = false;
					subqueryFound = false;
					itWasEnd = false;
				}
			}

			else {
				if (isInTable) {
					if (checkDetected) {
						reorganized = reorganized.concat(" " + temp);
						//System.out.println("%%");
						checkDetected = false;
					} else
						reorganized = reorganized.concat("\n" + temp);
				} else
					reorganized = reorganized.concat(" " + temp);
			}

			firstChar = false;
			//System.out.println(checkDetected);
		}

		/*
		 * nestingLevel=0; subqueryFound=false; perahpsSubqueryEnded=false;
		 * isInTable=false; itWasInTable=false;
		 */
		return reorganized;
	}

	/**
	 * This In-line string for granting that if you put spaces ,new lines or
	 * change case the string is correctly recognised. Then string is put in a
	 * standard fashion and comments are removed.
	 @since version 0.3.4
	 **/

	public String setStandard(String text) {
		StringBuilder tmp = new StringBuilder();
		char currentChar = '';
		char oldChar = '';
		boolean commentFound = false;

		for (int j = 0; j < text.length(); j++) {
			currentChar = text.charAt(j);
			if (currentChar == '\n')
				commentFound = false;
			if ((currentChar == '-' && oldChar == '-') || commentFound) {
				tmp.setCharAt(tmp.length() - 1, ' ');
				commentFound = true;
			} else
				tmp.append(currentChar);
			if (!commentFound)
				oldChar = currentChar;

		}
		text = tmp.toString();
		text = text.toLowerCase();
		StringTokenizer st = new StringTokenizer(text);
		text = "";
		while (st.hasMoreTokens()) {
			text = text.concat(st.nextToken() + " ");
		}

		return text;

	}
}