/* This file is part of MySQLConnectus by Sergio Decherchi

 Copyright (C) 2004 Sergio Decherchi

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 of 
 the License, or (at your option) any later version.
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details (gpl.txt).
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

package tools;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.BevelBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import mysqlconnectus.ImportTable;
import dbconnectivity.BufferedResultSet;

/**
 * A table inserted in a JInternalFrame based on the Vector model see source
 * 
 * @version 0.3
 * @author Sergio Decherchi
 */
/* VECTOR MODEL */
/*
 * Data structure that populates tables.
 * 
 * "->>" means "is composed by"
 * 
 * Vector currentDoc -->> |Vector page 1 ->> |Vector line 1 ->> |String cell 1
 * |Vector page 2 |Vector line 2 |String cell 2 |Vector page 3 | . . . |String
 * cell K (K is the number of cells for row) . . . .|Vector line M ( effective
 * number of tuples per page, must be <=N) . . |Vector page N (every page
 * contains 65 tuples)
 *  
 */

public class Table extends javax.swing.JInternalFrame {

	/** Table object */
	private javax.swing.JTable Data;

	/** Flag for table */
	private boolean isSaved = false;

	/** Current saved doc (the entire table) */
	private Vector currentDoc;

	/** Vector of strings of columns names */
	private Vector columnNames, columnType;

	/** Resource bundle for localization */
	private ResourceBundle Messages = ResourceBundle.getBundle("Lang/messages");

	/** Pointer to current table */
	final Table mytable = this;

	/** Current path */
	private File currentPath = new File(".");

	/**
	 * Cunstructor used for builidng a table by a saved table file or a buffered
	 * table.
	 * 
	 * @param savedDocx
	 *            is the current doc on which the table will be built
	 * @param columnNames
	 *            is the vector of strings of column names
	 * @param jdp
	 *            is the current instance of desktop on main window
	 * @param windowTitle
	 *            is the title of the window
	 * @param metadata
	 *            is a vector that contains the sql type of the columns
	 * @param currentParent
	 *            is a pointer to current frame parent
	 */
	public Table(Vector savedDocx, Vector columnNames, JDesktopPane jdp,
			String windowTitle, Vector metadata, final Component currentParent) {
		setFrameIcon(new ImageIcon("Resources/JTable.gif"));
		columnType = metadata;
		isSaved = true;
		final Vector savedDoc = savedDocx;
		this.columnNames = columnNames;
		currentDoc = savedDocx;
		final Vector currentColumnNames = columnNames;
		setTitle(windowTitle);
		setLocation(200, 0);
		this.setClosable(true);
		this.setMaximizable(true);
		this.setIconifiable(true);
		this.setResizable(true);

		JPanel panel = new JPanel();
		JButton print = new JButton(
				Messages.getString("Table.0"), new ImageIcon( //$NON-NLS-1$
						"Resources/print.gif")); //$NON-NLS-1$
		JButton save = new JButton(
				Messages.getString("Table.2"), new ImageIcon("Resources/save.gif")); //$NON-NLS-1$ //$NON-NLS-2$
		JButton exportToTxt = new JButton(
				Messages.getString("Table.4"), new ImageIcon( //$NON-NLS-1$
						"Resources/savetxt.gif")); //$NON-NLS-1$
		JButton exportToDB = new JButton(
				Messages.getString("Table.6"), new ImageIcon( //$NON-NLS-1$
						"Resources/savedb.gif")); //$NON-NLS-1$
		panel.add(print);
		panel.add(save);
		panel.add(exportToTxt);
		panel.add(exportToDB);
		getContentPane().add(panel, BorderLayout.SOUTH);

		print.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {
				try {
					Data.print();
				} catch (PrinterException e) {
					e.printStackTrace();
				}

			}

		});

		save.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				String tableName = ""; //$NON-NLS-1$
				String tablePath = ""; //$NON-NLS-1$

				JFileChooser chooser = new JFileChooser();
				chooser.setCurrentDirectory(currentPath); //$NON-NLS-1$
				chooser.addChoosableFileFilter(new TabFilter());
				chooser.setFileView(new FileIconViewForTab(new TabFilter(),
						new ImageIcon("Resources/JTable.gif"))); //$NON-NLS-1$

				int r = chooser.showSaveDialog(null);
				if (r == JFileChooser.APPROVE_OPTION) {
					tablePath = chooser.getSelectedFile().getAbsolutePath();
					tableName = chooser.getSelectedFile().getName();
					try {
						//System.out.println(tableName);

						if (!tablePath.endsWith(".dat")) { //$NON-NLS-1$
							tableName = tableName.concat(".dat"); //$NON-NLS-1$
							tablePath = tablePath.concat(".dat"); //$NON-NLS-1$
						}
						File fname = new File(tablePath);
						if (fname.exists()) {
							if (JOptionPane.showConfirmDialog(null, Messages
									.getString("FileManager.overwrite")) != 0) //$NON-NLS-1$
							{
								return;
							}
						}

						FileOutputStream fos = new FileOutputStream(tablePath);
						ObjectOutputStream oos = new ObjectOutputStream(fos);
						BufferedResultSet prs = new BufferedResultSet(oos,
								savedDoc, currentColumnNames,
								currentColumnNames.size(), savedDoc.size(),
								columnType);

						JOptionPane.showMessageDialog(null, Messages
								.getString("Table.17") //$NON-NLS-1$
								+ tableName, Messages
								.getString("MainFrame.Information"), 1); //$NON-NLS-1$

					} catch (FileNotFoundException fnfe) {
						JOptionPane
								.showMessageDialog(
										null,
										Messages.getString("HtmlHelpOnLine.7"), Messages.getString("MainFrame.Information"), 2); //$NON-NLS-1$ //$NON-NLS-2$
					} catch (IOException ioe) {
						JOptionPane.showMessageDialog(null, "IO Error!", //$NON-NLS-1$
								Messages.getString("MainFrame.Information"), 0); //$NON-NLS-1$
					}
					mytable.setTitle(tableName);
					currentPath = new File(tablePath);

				}

			}

		});
		final Vector cnames = columnNames;
		/*
		 * The table will be exported in filename.txt and metadata in
		 * filename@metadata.txt
		 */
		exportToTxt.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {
				String filename = null;
				JFileChooser chooser = new JFileChooser();
				chooser.setCurrentDirectory(currentPath); //$NON-NLS-1$
				int r = chooser.showSaveDialog(null);
				if (r == JFileChooser.APPROVE_OPTION) {

					filename = chooser.getSelectedFile().getAbsolutePath();
					FileManager fm = new FileManager();
					if (!filename.endsWith(".txt")) //$NON-NLS-1$
						filename = filename.concat(".txt"); //$NON-NLS-1$
					if (columnType != null && columnType.size() != 0)
						fm.saveFile(columnType, filename.substring(0, filename
								.length() - 4)
								+ "@metadata" + ".txt"); //$NON-NLS-1$ //$NON-NLS-2$
					if (!filename.endsWith(".txt")) //$NON-NLS-1$
						filename = filename.concat(".txt"); //$NON-NLS-1$
					fm.saveFile(cnames, savedDoc, filename);
					filename = chooser.getSelectedFile().getAbsolutePath();
					currentPath = new File(filename);
				}

			}
		});
		exportToDB.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent evt) {

				FileManager fm = new FileManager();

				if (!fm.saveFile(columnType, "temp@metadata.txt")) { //$NON-NLS-1$
					JOptionPane.showMessageDialog(null, Messages
							.getString("Table.29"), //$NON-NLS-1$
							Messages.getString("MainFrame.W"), 2); //$NON-NLS-1$

					return;
				}
				fm.saveFile(cnames, savedDoc, "temp.txt"); //$NON-NLS-1$
				ImportTable it = new ImportTable(currentParent);
				Object[] possibleValues = {Messages.getString("Table.32"), //$NON-NLS-1$
						Messages.getString("Table.33")}; //$NON-NLS-1$
				Object selectedValue = JOptionPane
						.showInputDialog(
								null,
								Messages.getString("Table.34"), Messages.getString("Table.35"), //$NON-NLS-1$ //$NON-NLS-2$
								JOptionPane.INFORMATION_MESSAGE, null,
								possibleValues, possibleValues[0]);
				if (selectedValue != null) {
					if (((String) selectedValue).equals(Messages
							.getString("Table.32"))) { //$NON-NLS-1$
						it.file = "temp.txt"; //$NON-NLS-1$
						it.metafile = "temp@metadata.txt"; //$NON-NLS-1$
						it.importFromTxtDirect();
					} else if (((String) selectedValue).equals(Messages
							.getString("Table.33"))) { //$NON-NLS-1$
						it.file = "temp.txt"; //$NON-NLS-1$
						it.metafile = "temp@metadata.txt"; //$NON-NLS-1$

						it.importCreateFromTxtDirect();
					} else {
						return;
					}
				}

			}
		});

		jdp.add(mytable);
		this.setVisible(true);

		initComponentsForBufferedTable();

	}

	/** Initialize components for tables generated by a BufferedResultSet */
	private void initComponentsForBufferedTable() {
		Data = new javax.swing.JTable();
		Data.setModel(getModelForSavedTable());
		Data.setGridColor(Color.BLUE);
		Data.doLayout();
		Data.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		JScrollPane scrollpane = new JScrollPane(Data);
		scrollpane.setBorder(new BevelBorder(BevelBorder.LOWERED));
		getContentPane().add(scrollpane);
		Data.setIntercellSpacing(new Dimension(1, 1));
		pack();

	}

	/** Get model for table generated by a BufferedResultSet */
	public TableModel getModelForSavedTable() {
		//Paste the vector in a correct fashion
		Vector tmp = new Vector();
		int i = currentDoc.size();
		for (int j = 0; j < i; j++)
			tmp.addAll((Vector) currentDoc.get(j));
		final Vector temp = tmp;

		return new AbstractTableModel() {

			public int getRowCount() {
				return temp.size();

			}

			public int getColumnCount() {

				return columnNames.size();
			}

			public String getColumnName(int column) {
				return (String) columnNames.get(column);
			}

			public Object getValueAt(int row, int column) {
				return (String) ((Vector) temp.get(row)).get(column);

			}

		};

	}

}